@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biewlib
  * @file        biewlib/twin.c
  * @brief       This file contains implementation of Text Window manager.
@@ -17,6 +17,11 @@
  * @bug         Limitation of twPrintF using
  * @todo        Accelerate windows interaction algorithm
 **/
+
+#include <string.h>
+#include <stdlib.h>
+#include <stdarg.h>
+#include <stdio.h>
 #include <limits.h>
 #ifdef __GNUC__
 #include <unistd.h>
@@ -34,15 +39,13 @@ const char TW_DN3D_FRAME[8] = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF";
 
 #ifndef NDEBUG
 #ifdef __TSC__
-#pragma save , call(inline=>on) /** must be GP in protmode */
+#pragma save , call(inline=>on) /** GPF in protmode */
 static void winInternalError( void ) = { 0xFF, 0xFF };
-#pragma restore
+#elif defined _MSC_VER
+static inline void winInternalError( void ) {};
 #else
-#ifdef __GNUC__
-#define winInternalError() asm(".short 0xFFFF\n");
+static inline void winInternalError( void ) { (void)0xFFFFFFFF; };
 #endif
-#endif
-
 static tBool __NEAR__ __FASTCALL__ test_win(TWindow *win)
 {
  tBool ret;
@@ -54,13 +57,11 @@ static tBool __NEAR__ __FASTCALL__ test_win(TWindow *win)
        *((void **)(win->saved.attrs + win->wsize)) == win->saved.attrs ? True : False;
  return ret;
 }
+#define CHECK_WINS(x) { if(!test_win(x)) winInternalError(); }
+#else
+#define CHECK_WINS(x)
 #endif
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stdio.h>
-
 #define IFLG_VISIBLE       0x00000001UL
 #define IFLG_ENABLED       0x00000002UL
 #define IFLG_CURSORBEENOFF 0x80000000UL
@@ -78,11 +79,107 @@ static TWindow *cursorwin = NULL;
 
 static unsigned long twin_flags = 0L;
 
-#ifndef NDEBUG
-#define CHECK_WINS(x) { if(!test_win(x)) winInternalError(); }
-#else
-#define CHECK_WINS(x)
-#endif
+#define __FIND_OVER(ret,win,x,y)\
+{\
+  TWindow *iter;\
+  tAbsCoord xx,yy;\
+  iter = head;\
+  ret = NULL;\
+  xx = win->X1+x;\
+  yy = win->Y1+y;\
+  while( iter && iter != win )\
+  {\
+    if((iter->iflags & IFLG_VISIBLE) == IFLG_VISIBLE)\
+    {\
+      if((yy >= iter->Y1) && (yy < iter->Y2) &&\
+         (xx >= iter->X1) && (xx < iter->X2))\
+                       ret = iter;\
+    }\
+    iter = iter->next;\
+  }\
+}
+
+#define IS_VALID_XY(win,x,y)\
+  ((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ?\
+   (tAbsCoord)x &&\
+   (tAbsCoord)x < win->wwidth-1 &&\
+   (tAbsCoord)y &&\
+   (tAbsCoord)y < win->wheight-1 :\
+   (tAbsCoord)x < win->wwidth &&\
+   (tAbsCoord)y < win->wheight)
+
+#define IS_VALID_X(win,x)\
+  ((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ?\
+   (tAbsCoord)x &&\
+   (tAbsCoord)x < win->wwidth-1:\
+   (tAbsCoord)x < win->wwidth)
+
+#define IS_VALID_Y(win,y)\
+  ((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ?\
+   (tAbsCoord)y &&\
+   (tAbsCoord)y < win->wheight-1 :\
+   (tAbsCoord)y < win->wheight)
+
+static tBool __NEAR__ __FASTCALL__ isOverlapped(TWindow *win)
+{
+  TWindow *iter;
+  tBool ret = False;
+  iter = head;
+  while( iter && iter != win )
+  {
+    if((iter->iflags & IFLG_VISIBLE) == IFLG_VISIBLE)
+    {
+      if(!((iter->Y2 <= win->Y1) || (iter->Y1 >= win->Y2) ||
+           (iter->X2 <= win->X1) || (iter->X1 >= win->X2)))
+      {
+        ret = True;
+        break;
+      }
+    }
+    iter = iter->next;
+  }
+  return ret;
+}
+
+/*
+   In most cases all output is being performed into non overlapped windows.
+   We should turn on special optimization for such windows, though overlapped
+   windows will seem relatively slower. There are additional possibilities to
+   accelerate all output by computing non overlapped lines of window or parts
+   of lines, but I don't want to do such complex task.
+*/
+#define __TOPMOST(win) (!isOverlapped(win))
+
+#define __ATHEAD(win)\
+{\
+  if(head) ((TWindow *)win)->next = head;\
+  head = (TWindow *)win;\
+}
+
+#define __ATWIN(win,prev)\
+{\
+  if(!prev) __ATHEAD(win)\
+  else\
+  {\
+     ((TWindow *)win)->next = ((TWindow *)prev)->next;\
+     ((TWindow *)prev)->next = (TWindow *)win;\
+  }\
+}
+
+#define __AT_POINT(iter,x,y)\
+{\
+  iter = head;\
+  while( iter )\
+  {\
+    if((iter->iflags & IFLG_VISIBLE) == IFLG_VISIBLE)\
+    {\
+      if(((tAbsCoord)y >= iter->Y1) && ((tAbsCoord)y < iter->Y2) &&\
+         ((tAbsCoord)x >= iter->X1) && ((tAbsCoord)x < iter->X2))\
+                       break;\
+    }\
+    iter = iter->next;\
+  }\
+}
 
 static int c_type = -1;
 void __FASTCALL__ twSetCursorType(int type)
@@ -118,6 +215,7 @@ void __FASTCALL__ twInit( unsigned long vio_flags, unsigned long twin_flgs )
 void __FASTCALL__ twDestroy( void )
 {
   twSetCursorType(TW_CUR_NORM);
+  twcDestroyClassSet();
   __term_keyboard();
   __term_vio();
 }
@@ -385,22 +483,6 @@ static void __NEAR__ __FASTCALL__ __unlistwin(TWindow *win)
   }
 }
 
-#define __ATHEAD(win)\
-{\
-  if(head) ((TWindow *)win)->next = head;\
-  head = (TWindow *)win;\
-}
-
-#define __ATWIN(win,prev)\
-{\
-  if(!prev) __ATHEAD(win)\
-  else\
-  {\
-     ((TWindow *)win)->next = ((TWindow *)prev)->next;\
-     ((TWindow *)prev)->next = (TWindow *)win;\
-  }\
-}
-
 static TWindow * __NEAR__ __FASTCALL__ __prevwin(TWindow *win)
 {
   TWindow *ret = NULL;
@@ -430,21 +512,6 @@ static TWindow * __NEAR__ __FASTCALL__ __findcursorablewin(void)
   return ret;
 }
 
-#define __AT_POINT(iter,x,y)\
-{\
-  iter = head;\
-  while( iter )\
-  {\
-    if((iter->iflags & IFLG_VISIBLE) == IFLG_VISIBLE)\
-    {\
-      if(((tAbsCoord)y >= iter->Y1) && ((tAbsCoord)y < iter->Y2) &&\
-         ((tAbsCoord)x >= iter->X1) && ((tAbsCoord)x < iter->X2))\
-                       break;\
-    }\
-    iter = iter->next;\
-  }\
-}
-
 TWindow * __FASTCALL__ twGetWinAtPos(tAbsCoord x,tAbsCoord y)
 {
   TWindow *ret;
@@ -455,47 +522,6 @@ TWindow * __FASTCALL__ twGetWinAtPos(tAbsCoord x,tAbsCoord y)
   return ret;
 }
 
-#define __FIND_OVER(ret,win,x,y)\
-{\
-  TWindow *iter;\
-  tAbsCoord xx,yy;\
-  iter = head;\
-  ret = NULL;\
-  xx = win->X1+x;\
-  yy = win->Y1+y;\
-  while( iter && iter != win )\
-  {\
-    if((iter->iflags & IFLG_VISIBLE) == IFLG_VISIBLE)\
-    {\
-      if((yy >= iter->Y1) && (yy < iter->Y2) &&\
-         (xx >= iter->X1) && (xx < iter->X2))\
-                       ret = iter;\
-    }\
-    iter = iter->next;\
-  }\
-}
-
-#define IS_VALID_XY(win,x,y)\
-  ((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ?\
-   (tAbsCoord)x &&\
-   (tAbsCoord)x < win->wwidth-1 &&\
-   (tAbsCoord)y &&\
-   (tAbsCoord)y < win->wheight-1 :\
-   (tAbsCoord)x < win->wwidth &&\
-   (tAbsCoord)y < win->wheight)
-
-#define IS_VALID_X(win,x)\
-  ((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ?\
-   (tAbsCoord)x &&\
-   (tAbsCoord)x < win->wwidth-1:\
-   (tAbsCoord)x < win->wwidth)
-
-#define IS_VALID_Y(win,y)\
-  ((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ?\
-   (tAbsCoord)y &&\
-   (tAbsCoord)y < win->wheight-1 :\
-   (tAbsCoord)y < win->wheight)
-
 void __FASTCALL__ twCvtWinCoords(TWindow *win,tRelCoord x, tRelCoord y,tAbsCoord *xa,tAbsCoord *ya)
 {
   if(win->flags & TWS_FRAMEABLE) { x++; y++; }
@@ -888,19 +914,26 @@ static void __NEAR__ __FASTCALL__ screen2win(TWindow *win)
         __MsSetState(False);
       }
     }
-    for(i = 0;i < win->wheight;i++)
+    if(win->wwidth == tvioWidth && !win->X1)
     {
-      tAbsCoord iny;
-      iny = win->Y1+i;
-      if(iny <= tvioHeight)
+      __vioReadBuff(0, win->Y1, &win->saved, win->wwidth*win->wheight);
+    }
+    else
+    {
+      for(i = 0;i < win->wheight;i++)
       {
-        idx = i*win->wwidth;
-        it.chars = &win->saved.chars[idx];
-        it.oem_pg = &win->saved.oem_pg[idx];
-        it.attrs = &win->saved.attrs[idx];
-        __vioReadBuff(inx,iny,&it,lwidth);
+        tAbsCoord iny;
+        iny = win->Y1+i;
+        if(iny <= tvioHeight)
+        {
+          idx = i*win->wwidth;
+          it.chars = &win->saved.chars[idx];
+          it.oem_pg = &win->saved.oem_pg[idx];
+          it.attrs = &win->saved.attrs[idx];
+          __vioReadBuff(inx,iny,&it,lwidth);
+        }
+        else break;
       }
-      else break;
     }
     if(is_hidden) __MsSetState(True);
   }
@@ -960,13 +993,12 @@ void __FASTCALL__ twSnapShot(TWindow *win) /**< for snapshot */
 
 static void __NEAR__ __FASTCALL__ savedwin2screen(TWindow *win)
 {
-  unsigned i,j;
+  unsigned i,j, tidx;
   tvioBuff accel;
   t_vchar chars[__TVIO_MAXSCREENWIDTH];
   t_vchar oem_pg[__TVIO_MAXSCREENWIDTH];
   ColorAttr attrs[__TVIO_MAXSCREENWIDTH];
-  tBool ms_vis;
-  tBool is_hidden = False;
+  tBool ms_vis, is_hidden = False, is_top;
   accel.chars = chars;
   accel.oem_pg = oem_pg;
   accel.attrs = attrs;
@@ -983,20 +1015,34 @@ static void __NEAR__ __FASTCALL__ savedwin2screen(TWindow *win)
         __MsSetState(False);
       }
     }
-    for(i = 0;i < win->wheight;i++)
+    is_top = __TOPMOST(win);
+    if(is_top && win->wwidth == tvioWidth && !win->X1)
     {
-      tAbsCoord outx,outy;
-      unsigned nwidth;
-      for(j = 0;j < win->wwidth;j++)
+      /* Special case of redrawing window interior at one call */
+      __vioWriteBuff(0, win->Y1, &win->saved, win->wwidth*win->wheight);
+    }
+    else
+    {
+      for(i = 0;i < win->wheight;i++)
       {
-        restorescreenchar(win,j+1,i+1,&accel);
+        tAbsCoord outx,outy;
+        unsigned nwidth;
+        if(!is_top)
+          for(j = 0;j < win->wwidth;j++) restorescreenchar(win,j+1,i+1,&accel);
+        else
+        {
+           tidx = i*win->wwidth;
+           accel.chars = &win->saved.chars[tidx];
+           accel.attrs = &win->saved.attrs[tidx];
+           accel.oem_pg = &win->saved.oem_pg[tidx];
+        }
+        outx = win->X1;
+        outy = win->Y1+i;
+        nwidth = win->wwidth;
+        if(outx + nwidth > tvioWidth) nwidth = tvioWidth > outx ? tvioWidth - outx : 0;
+        if(outy <= tvioHeight && nwidth)
+            __vioWriteBuff(outx,outy,&accel,nwidth);
       }
-      outx = win->X1;
-      outy = win->Y1+i;
-      nwidth = win->wwidth;
-      if(outx + nwidth > tvioWidth) nwidth = tvioWidth > outx ? tvioWidth - outx : 0;
-      if(outy <= tvioHeight && nwidth)
-          __vioWriteBuff(outx,outy,&accel,nwidth);
     }
     if(is_hidden) __MsSetState(True);
     CHECK_WINS(win);
@@ -1005,15 +1051,14 @@ static void __NEAR__ __FASTCALL__ savedwin2screen(TWindow *win)
 
 static void __NEAR__ __FASTCALL__ updatescreen(TWindow *win,tBool full_area)
 {
-  unsigned i,j;
+  unsigned i,j, tidx;
   tAbsCoord xs,xe,ys,ye,cx,rw;
   unsigned aoff;
   tvioBuff accel,it;
   t_vchar chars[__TVIO_MAXSCREENWIDTH];
   t_vchar oem_pg[__TVIO_MAXSCREENWIDTH];
   ColorAttr attrs[__TVIO_MAXSCREENWIDTH];
-  tBool ms_vis;
-  tBool is_hidden = False;
+  tBool ms_vis, is_hidden = False, is_top;
   accel.chars = chars;
   accel.oem_pg = oem_pg;
   accel.attrs = attrs;
@@ -1038,35 +1083,43 @@ static void __NEAR__ __FASTCALL__ updatescreen(TWindow *win,tBool full_area)
         __MsSetState(False);
       }
     }
-    xs = ys = 0;
-    xe = win->wwidth;
-    ye = win->wheight;
-    cx = win->X1;
-    rw = win->wwidth;
-    aoff = 0;
-    if((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE && (!full_area))
+    is_top = __TOPMOST(win);
+    if(is_top && full_area && win->wwidth == tvioWidth && !win->X1 && (win->iflags & IFLG_ENABLED) == IFLG_ENABLED)
     {
-        xs++; xe--; ys++; ye--;
-        cx++; rw-=2;
-        aoff = 1;
+      /* Special case of redrawing window interior at one call */
+      __vioWriteBuff(0, win->Y1, &win->body, win->wwidth*win->wheight);
     }
-    for(i = ys;i < ye;i++)
+    else
     {
-      tAbsCoord outy;
-      for(j = xs;j < xe;j++)
+      xs = ys = 0;
+      xe = win->wwidth;
+      ye = win->wheight;
+      cx = win->X1;
+      rw = win->wwidth;
+      aoff = 0;
+      if((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE && (!full_area))
       {
-        updatescreenchar(win,j+1,i+1,&accel);
+          xs++; xe--; ys++; ye--;
+          cx++; rw-=2;
+          aoff = 1;
       }
-      if((win->iflags & IFLG_ENABLED) == IFLG_ENABLED)
+      for(i = ys;i < ye;i++)
       {
-        outy = win->Y1+i;
-        if(cx + rw > tvioWidth) rw = tvioWidth > cx ? tvioWidth - cx : 0;
-        if(outy <= tvioHeight && rw)
+        tAbsCoord outy;
+        if(!is_top)
+          for(j = xs;j < xe;j++) updatescreenchar(win,j+1,i+1,&accel);
+        if((win->iflags & IFLG_ENABLED) == IFLG_ENABLED)
         {
-          it.chars = &accel.chars[aoff];
-          it.oem_pg = &accel.oem_pg[aoff];
-          it.attrs = &accel.attrs[aoff];
-          __vioWriteBuff(cx,outy,&it,rw);
+          outy = win->Y1+i;
+          if(cx + rw > tvioWidth) rw = tvioWidth > cx ? tvioWidth - cx : 0;
+          if(outy <= tvioHeight && rw)
+          {
+            tidx = i*win->wwidth+aoff;
+            it.chars = is_top ? &win->body.chars[tidx] : &accel.chars[aoff];
+            it.oem_pg = is_top ? &win->body.oem_pg[tidx] : &accel.oem_pg[aoff];
+            it.attrs = is_top ? &win->body.attrs[tidx] : &accel.attrs[aoff];
+            __vioWriteBuff(cx,outy,&it,rw);
+          }
         }
       }
     }
@@ -1077,14 +1130,13 @@ static void __NEAR__ __FASTCALL__ updatescreen(TWindow *win,tBool full_area)
 
 static void __NEAR__ __FASTCALL__ updatescreenpiece(TWindow *win,tRelCoord stx,tRelCoord endx,tRelCoord y)
 {
-  unsigned i,line;
+  unsigned i,line, tidx;
   tAbsCoord _stx,_endx;
   tvioBuff accel,it;
   t_vchar chars[__TVIO_MAXSCREENWIDTH];
   t_vchar oem_pg[__TVIO_MAXSCREENWIDTH];
   ColorAttr attrs[__TVIO_MAXSCREENWIDTH];
-  tBool ms_vis;
-  tBool is_hidden = False;
+  tBool ms_vis, is_hidden = False, is_top;
   accel.chars = chars;
   accel.oem_pg = oem_pg;
   accel.attrs = attrs;
@@ -1101,9 +1153,13 @@ static void __NEAR__ __FASTCALL__ updatescreenpiece(TWindow *win,tRelCoord stx,t
     }
     _stx = min(stx,win->wwidth);
     _endx = min(endx,win->wwidth);
-    if(_stx < _endx)
-      for(i = stx;i < endx;i++)
-        updatescreenchar(win,i+1,y,&accel);
+    is_top = __TOPMOST(win);
+    if(!is_top)
+    {
+      if(_stx < _endx)
+        for(i = stx;i < endx;i++)
+          updatescreenchar(win,i+1,y,&accel);
+    }
     if((win->iflags & IFLG_ENABLED) == IFLG_ENABLED)
     {
       tAbsCoord outx;
@@ -1113,9 +1169,10 @@ static void __NEAR__ __FASTCALL__ updatescreenpiece(TWindow *win,tRelCoord stx,t
       if(outx + rw > tvioWidth) rw = tvioWidth > outx ? tvioWidth - outx : 0;
       if(line <= tvioHeight && rw)
       {
-        it.chars = &accel.chars[_stx];
-        it.oem_pg = &accel.oem_pg[_stx];
-        it.attrs = &accel.attrs[_stx];
+        tidx = (y-1)*win->wwidth+_stx;
+        it.chars = is_top ? &win->body.chars[tidx] : &accel.chars[_stx];
+        it.oem_pg = is_top ? &win->body.oem_pg[tidx] : &accel.oem_pg[_stx];
+        it.attrs = is_top ? &win->body.attrs[tidx] : &accel.attrs[_stx];
         __vioWriteBuff(outx,line,&it,rw);
       }
     }
@@ -1126,13 +1183,12 @@ static void __NEAR__ __FASTCALL__ updatescreenpiece(TWindow *win,tRelCoord stx,t
 
 static void __NEAR__ __FASTCALL__ updatewinmem(TWindow *win)
 {
-  unsigned i,j;
+  unsigned i,j,tidx;
   tvioBuff accel;
   t_vchar chars[__TVIO_MAXSCREENWIDTH];
   t_vchar oem_pg[__TVIO_MAXSCREENWIDTH];
   ColorAttr attrs[__TVIO_MAXSCREENWIDTH];
-  tBool ms_vis;
-  tBool is_hidden = False;
+  tBool ms_vis, is_hidden = False, is_top;  
   accel.chars = chars;
   accel.oem_pg = oem_pg;
   accel.attrs = attrs;
@@ -1149,19 +1205,36 @@ static void __NEAR__ __FASTCALL__ updatewinmem(TWindow *win)
         __MsSetState(False);
       }
     }
-    for(i = 0;i < win->wheight;i++)
+    is_top = __TOPMOST(win);
+    if(is_top && win->wwidth == tvioWidth && !win->X1)
     {
-      tAbsCoord inx,iny;
-      unsigned lwidth;
-      lwidth = win->wwidth;
-      inx = win->X1;
-      iny = win->Y1+i;
-      if(inx + lwidth > tvioWidth) lwidth = tvioWidth > inx ? tvioWidth - inx : 0;
-      if(iny <= tvioHeight && lwidth)
-        __vioReadBuff(inx,iny,&accel,lwidth);
-      for(j = 0;j < win->wwidth;j++)
+      /* Special case of redrawing window interior at one call */
+      __vioReadBuff(0, win->Y1, &win->saved, win->wwidth*win->wheight);
+    }
+    else
+    {
+      for(i = 0;i < win->wheight;i++)
       {
-        updatewinmemcharfromscreen(win,j,i,&accel);
+        tAbsCoord inx,iny;
+        unsigned lwidth;
+        lwidth = win->wwidth;
+        inx = win->X1;
+        iny = win->Y1+i;
+        if(inx + lwidth > tvioWidth) lwidth = tvioWidth > inx ? tvioWidth - inx : 0;
+        if(iny <= tvioHeight && lwidth)
+        {
+          if(is_top)
+          {
+            tidx = i*win->wwidth;
+            accel.chars = &win->saved.chars[tidx];
+            accel.attrs = &win->saved.attrs[tidx];
+            accel.oem_pg = &win->saved.oem_pg[tidx];
+          }
+          __vioReadBuff(inx,iny,&accel,lwidth);
+        }
+        if(!is_top)
+          for(j = 0;j < win->wwidth;j++)
+             updatewinmemcharfromscreen(win,j,i,&accel);
       }
     }
     if(is_hidden) __MsSetState(True);
@@ -1280,10 +1353,10 @@ static void __NEAR__ __FASTCALL__ make_frame( void )
    slen = strlen(active->Title);
    if(slen > w) slen = w;
    stx = calc_title_off(active->TitleMode,w,slen);
-   for(i = 0;i < slen;i++) 
-   { 
-      iGotoXY(stx+i,1); 
-      wputc(active->Title[i],active->title.system,False); 
+   for(i = 0;i < slen;i++)
+   {
+      iGotoXY(stx+i,1);
+      wputc(active->Title[i],active->title.system,False);
    }
  }
  if(active->Footer)
@@ -1293,10 +1366,10 @@ static void __NEAR__ __FASTCALL__ make_frame( void )
    slen = strlen(active->Footer);
    if(slen > w) slen = w;
    stx = calc_title_off(active->FooterMode,w,slen);
-   for(i = 0;i < slen;i++) 
-   { 
-      iGotoXY(stx+i,h); 
-      wputc(active->Footer[i],active->footer.system,False); 
+   for(i = 0;i < slen;i++)
+   {
+      iGotoXY(stx+i,h);
+      wputc(active->Footer[i],active->footer.system,False);
    }
  }
  updatescreenpiece(active,0,active->wwidth,1);
@@ -1360,14 +1433,37 @@ TWindow * __FASTCALL__ twCreateWin(tAbsCoord x1, tAbsCoord y1, tAbsCoord width,
   return win;
 }
 
+TWindow *  __FASTCALL__ twCreateWinEx(tAbsCoord x1_, tAbsCoord y1_,
+                                      tAbsCoord width, tAbsCoord height,
+                                      unsigned flags, TWindow *parent,
+                                      const char *classname)
+{
+  TWindow *ret;
+  UNUSED(parent);
+  if((ret = twCreateWin(x1_, y1_, width, height, flags)) != NULL)
+  {
+    TwClass *cls;
+    cls = twcFindClass(classname);
+    if(cls)
+    {
+      ret->method = cls->method;
+      ret->class_flags = cls->flags;
+    }
+    if(ret->method) ((twClassFunc)(ret->method))(ret,WM_CREATE,0L,NULL);
+  }
+  return ret;
+}
+
+
 void __FASTCALL__ twShowWin(TWindow* w)
 {
+  if(w->method) ((twClassFunc)(w->method))(w,WM_SHOW,0L,NULL);
   if(!(w->iflags & IFLG_VISIBLE) == IFLG_VISIBLE)
   {
     w->iflags |= IFLG_VISIBLE;
     __unlistwin(w);
     __ATHEAD(w);
-    screen2win(w);
+    updatewinmem(w);
     updatescreen(w,True);
     if((w->flags & TWS_CURSORABLE) == TWS_CURSORABLE)
     {
@@ -1379,11 +1475,12 @@ void __FASTCALL__ twShowWin(TWindow* w)
 
 void __FASTCALL__ twShowWinOnTop(TWindow* w)
 {
+  if(w->method) ((twClassFunc)(w->method))(w,WM_TOPSHOW,0L,NULL);
   if((w->iflags & IFLG_VISIBLE) == IFLG_VISIBLE) twHideWin(w);
   w->iflags |= IFLG_VISIBLE;
   __unlistwin(w);
   __ATHEAD(w);
-  updatewinmem(w);
+  screen2win(w);
   updatescreen(w,True);
   if((w->flags & TWS_CURSORABLE) == TWS_CURSORABLE)
   {
@@ -1394,6 +1491,7 @@ void __FASTCALL__ twShowWinOnTop(TWindow* w)
 
 void __FASTCALL__ twShowWinBeneath(TWindow* w,TWindow *prev)
 {
+  if(w->method) ((twClassFunc)(w->method))(w,WM_SHOWBENEATH,0L,prev);
   if((w->iflags & IFLG_VISIBLE) == IFLG_VISIBLE) twHideWin(w);
   w->iflags |= IFLG_VISIBLE;
   __unlistwin(w);
@@ -1404,6 +1502,7 @@ void __FASTCALL__ twShowWinBeneath(TWindow* w,TWindow *prev)
 
 void __FASTCALL__ twHideWin(TWindow *w)
 {
+  if(w->method) ((twClassFunc)(w->method))(w,WM_HIDE,0L,NULL);
   if(cursorwin == w) twSetCursorType(TW_CUR_OFF);
   savedwin2screen(w);
   w->iflags &= ~IFLG_VISIBLE;
@@ -1411,6 +1510,7 @@ void __FASTCALL__ twHideWin(TWindow *w)
 
 void __FASTCALL__ twDestroyWin(TWindow *win)
 {
+  if(win->method) ((twClassFunc)(win->method))(win,WM_DESTROY,0L,NULL);
   twHideWin(win);
   if(win->Title) PFREE(win->Title);
   if(win->Footer) PFREE(win->Footer);
@@ -1456,7 +1556,7 @@ unsigned          __FASTCALL__ twGetWinHeight(TWindow *win)
 
 unsigned          __FASTCALL__ twGetClientWidth(TWindow *win)
 {
-  return win->flags & TWS_FRAMEABLE ? win->wwidth-2 : win->wwidth; 
+  return win->flags & TWS_FRAMEABLE ? win->wwidth-2 : win->wwidth;
 }
 
 unsigned          __FASTCALL__ twGetClientHeight(TWindow *win)
@@ -1546,7 +1646,7 @@ void __FASTCALL__ twResizeWin(TWindow *win,tAbsCoord width,tAbsCoord height)
       memcpy(&newbody.chars[to],&win->body.chars[from],ncopy);
       memcpy(&newbody.oem_pg[to],&win->body.oem_pg[from],ncopy);
       memcpy(&newbody.attrs[to],&win->body.attrs[from],ncopy);
-      if(oldw < width) 
+      if(oldw < width)
       {
         idx = to+ncopy-delta;
         memset(&newbody.chars[idx],TWC_DEF_FILLER,fillsize);
@@ -1821,11 +1921,8 @@ int __FASTCALL__ twPutS(const char *str)
     if(IS_VALID_XY(active,cx,cy))
     {
         active->body.chars[vidx] = ch;
-        if((active->flags & TWS_NLSOEM) == TWS_NLSOEM)
-	{
-          if(NLS_IS_OEMPG(as_oem)) 
-            active->body.oem_pg[vidx] = as_oem;
-	}
+        if((active->flags & TWS_NLSOEM) == TWS_NLSOEM && NLS_IS_OEMPG(as_oem))
+             active->body.oem_pg[vidx] = as_oem;
 	else active->body.oem_pg[vidx] = 0;
         active->body.attrs[vidx++] = active->text.system;
         active->cur_x++;
@@ -1964,7 +2061,7 @@ void __FASTCALL__ twRefreshPiece(TWindow *win,tRelCoord stx,tRelCoord endx,tRelC
 void __FASTCALL__ twRefreshWin(TWindow *win)
 {
   win->iflags |= IFLG_ENABLED;
-  updatescreen(win,False);
+  updatescreen(win,(win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE ? False : True);
   paint_cursor();
 }
 
@@ -2058,7 +2155,7 @@ void __FASTCALL__ twScrollWinLf(TWindow *win,tRelCoord xpos, unsigned npos)
   accel.attrs = attrs;
   lim = xpos > npos ? xpos - npos : 0;
   wheight = win->wheight;
-  if((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE) 
+  if((win->flags & TWS_FRAMEABLE) == TWS_FRAMEABLE)
   {
     wheight-=2;
   }
