@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/elf386.h
  * @brief       This file defines standard ELF types, structures, and macros.
@@ -17,6 +17,8 @@
  *              License along with the GNU C Library; see the file COPYING.LIB.  If not,
  *              write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *              Boston, MA 02111-1307, USA.
+ * @remark	portions taken from FreeBSD (sys/elf*.h), and updates from
+ *              current draft (http://www.caldera.com/developers/gabi/)
  * @note        Requires POSIX compatible development system
  *
  * @author      GNU FSF
@@ -60,8 +62,34 @@
 
 #define EI_VERSION	6		/**< File version */
 
-#define EI_PAD		7		/**< Start of padding bytes */
+#define EI_OSABI	7		/**< Operating system / ABI identification */
+#define ELFOSABI_SYSV		0	/**< UNIX System V ABI */
+#define ELFOSABI_NONE		ELFOSABI_SYSV	/**< symbol used in old spec */
+#define ELFOSABI_HPUX		1	/**< HP-UX operating system */
+#define ELFOSABI_NETBSD		2	/**< NetBSD */
+#define ELFOSABI_LINUX		3	/**< GNU/Linux */
+#define ELFOSABI_HURD		4	/**< GNU/Hurd */
+#define ELFOSABI_86OPEN		5	/**< 86Open common IA32 ABI */
+#define ELFOSABI_SOLARIS	6	/**< Solaris */
+#define ELFOSABI_MONTEREY	7	/**< Monterey */
+#define ELFOSABI_IRIX		8	/**< IRIX */
+#define ELFOSABI_FREEBSD	9	/**< FreeBSD */
+#define ELFOSABI_TRU64		10	/**< TRU64 UNIX */
+#define ELFOSABI_MODESTO	11	/**< Novell Modesto */
+#define ELFOSABI_OPENBSD	12	/**< OpenBSD */
+#define ELFOSABI_ARM		97	/**< ARM */
+#define ELFOSABI_STANDALONE	255	/**< Standalone (embedded) application */
 
+#define EI_ABIVERSION	8	/**< ABI version */
+#define OLD_EI_BRAND	8	/**< Start of architecture identification. */
+#define EI_PAD		9	/**< Start of padding (per SVR4 ABI). */
+#define EI_NIDENT	16	/**< Size of e_ident array. */
+
+/* e_ident */
+#define IS_ELF(e_ident)	(e_ident[EI_MAG0] == ELFMAG0 && \
+			 e_ident[EI_MAG1] == ELFMAG1 && \
+			 e_ident[EI_MAG2] == ELFMAG2 && \
+			 e_ident[EI_MAG3] == ELFMAG3)
 
 /** Values for e_type, which identifies the object file type */
 
@@ -70,7 +98,8 @@
 #define ET_EXEC		2		/**< Executable file */
 #define ET_DYN		3		/**< Shared object file */
 #define ET_CORE		4		/**< Core file */
-#define	ET_NUM		5		/**< Number of defined types.  */
+#define ET_LOOS		0xFE00U		/**< OS-specific */
+#define ET_HIOS		0xFEFFU		/**< OS-specific */
 #define ET_LOPROC	0xFF00U		/**< Processor-specific */
 #define ET_HIPROC	0xFFFFU		/**< Processor-specific */
 
@@ -78,24 +107,83 @@
 
 #define EM_NONE		0	/**< No machine */
 #define EM_M32		1	/**< AT&T WE 32100 */
-#define EM_SPARC	2	/**< SUN SPARC */
+#define EM_SPARC	2	/**< Sun SPARC */
 #define EM_386		3	/**< Intel 80386 */
-#define EM_68K		4	/**< Motorola m68k family */
-#define EM_88K		5	/**< Motorola m88k family */
+#define EM_68K		4	/**< Motorola m68k */
+#define EM_88K		5	/**< Motorola m88k */
+				/* 6 reserved, was EM_486 */
 #define EM_860		7	/**< Intel 80860 */
-#define EM_MIPS		8	/**< MIPS R3000 (officially, big-endian only) */
-
-#define EM_MIPS_RS4_BE 10	/**< MIPS R4000 big-endian */
-
-#define EM_SPARC64     11	/**< SPARC v9 (not official) 64-bit */
-
-#define EM_PARISC      15	/**< HPPA */
-
-#define EM_SPARC32PLUS 18	/**< Sun's "v8plus" */
-
-#define EM_PPC	       20	/**< PowerPC */
-
-#define EM_SH	       42	/**< Hitachi SH */
+#define EM_MIPS		8	/**< MIPS I */
+#define EM_S370		9	/**< IBM System/370 */
+#define EM_MIPS_RS3_LE	10	/**< MIPS R3000 little-endian */
+				/**< 11-14 reserved */
+#define EM_PARISC	15	/**< HP PA-RISC */
+				/**< 16 reserved */
+#define	EM_VPP500	17	/**< Fujitsu VPP500 */
+#define EM_SPARC32PLUS	18	/**< Sun SPARC v8plus */
+#define EM_960		19	/**< Intel 80960 */
+#define EM_PPC		20	/**< PowerPC */
+#define EM_PPC64	21	/**< PowerPC 64-bit */
+#define	EM_S390		22	/**< IBM System/390 */
+				/**< 23-35 reserved */
+#define	EM_V800		36	/**< NEC V800 */
+#define	EM_FR20		37	/**< Fujitsu FR20 */
+#define	EM_RH32		38	/**< TRW RH-32 */
+#define	EM_RCE		39	/**< Motorola RCE */
+#define	EM_ARM		40	/**< Advanced RISC Machines ARM */
+#define	EM_ALPHA	41	/**< Digital Alpha */
+#define	EM_SH		42	/**< Hitachi SH */
+#define	EM_SPARCV9	43	/**< SPARC Version 9 64-bit */
+#define	EM_TRICORE	44	/**< Siemens TriCore embedded processor */
+#define	EM_ARC		45	/**< Argonaut RISC Core, Argonaut Technologies Inc. */
+#define	EM_H8_300	46	/**< Hitachi H8/300 */
+#define	EM_H8_300H	47	/**< Hitachi H8/300H */
+#define	EM_H8S		48	/**< Hitachi H8S */
+#define	EM_H8_500	49	/**< Hitachi H8/500 */
+#define	EM_IA_64	50	/**< Intel IA-64 processor architecture */
+#define	EM_MIPS_X	51	/**< Stanford MIPS-X */
+#define	EM_COLDFIRE	52	/**< Motorola ColdFire */
+#define	EM_68HC12	53	/**< Motorola M68HC12 */
+#define	EM_MMA		54	/**< Fujitsu MMA Multimedia Accelerator */
+#define	EM_PCP		55	/**< Siemens PCP */
+#define	EM_NCPU		56	/**< Sony nCPU embedded RISC processor */
+#define	EM_NDR1		57	/**< Denso NDR1 microprocessor */
+#define	EM_STARCORE	58	/**< Motorola Star*Core processor */
+#define	EM_ME16		59	/**< Toyota ME16 processor */
+#define	EM_ST100	60	/**< STMicroelectronics ST100 processor */
+#define	EM_TINYJ	61	/**< Advanced Logic Corp. TinyJ embedded processor family */
+#define	EM_X86_64	62	/**< AMD x86-64 architecture */
+#define	EM_PDSP		63	/**< Sony DSP Processor */
+				/**< 64-65 reserved */
+#define	EM_FX66		66	/**< Siemens FX66 microcontroller */
+#define	EM_ST9PLUS	67	/**< STMicroelectronics ST9+ 8/16 bit microcontroller */
+#define	EM_ST7		68	/**< STMicroelectronics ST7 8-bit microcontroller */
+#define	EM_68HC16	69	/**< Motorola MC68HC16 Microcontroller */
+#define	EM_68HC11	70	/**< Motorola MC68HC11 Microcontroller */
+#define	EM_68HC08	71	/**< Motorola MC68HC08 Microcontroller */
+#define	EM_68HC05	72	/**< Motorola MC68HC05 Microcontroller */
+#define	EM_SVX		73	/**< Silicon Graphics SVx */
+#define	EM_ST19		74	/**< STMicroelectronics ST19 8-bit microcontroller */
+#define	EM_VAX		75	/**< Digital VAX */
+#define	EM_CRIS		76	/**< Axis Communications 32-bit embedded processor */
+#define	EM_JAVELIN	77	/**< Infineon Technologies 32-bit embedded processor */
+#define	EM_FIREPATH	78	/**< Element 14 64-bit DSP Processor */
+#define	EM_ZSP		79	/**< LSI Logic 16-bit DSP Processor */
+#define	EM_MMIX		80	/**< Donald Knuth's educational 64-bit processor */
+#define	EM_HUANY	81	/**< Harvard University machine-independent object files */
+#define	EM_PRISM	82	/**< SiTera Prism */
+#define	EM_AVR		83	/**< Atmel AVR 8-bit microcontroller */
+#define	EM_FR30		84	/**< Fujitsu FR30 */
+#define	EM_D10V		85	/**< Mitsubishi D10V */
+#define	EM_D30V		86	/**< Mitsubishi D30V */
+#define	EM_V850		87	/**< NEC v850 */
+#define	EM_M32R		88	/**< Mitsubishi M32R */
+#define	EM_MN10300	89	/**< Matsushita MN10300 */
+#define	EM_MN10200	90	/**< Matsushita MN10200 */
+#define	EM_PJ		91	/**< picoJava */
+#define	EM_OPENRISC	92	/**< OpenRISC 32-bit embedded processor */
+#define	EM_ARC_A5	93	/**< ARC Cores Tangent-A5 */
+#define	EM_XTENSA	94	/**< Tensilica Xtensa Architecture */
 
 /** If it is necessary to assign new unofficial EM_* values, please pick large
    random numbers (0x8523, 0xa7f2, etc.) to minimize the chances of collision
@@ -108,21 +196,12 @@
 /** Cygnus PowerPC ELF backend.  Written in the absence of an ABI.  */
 #define EM_CYGNUS_POWERPC 0x9025U
 
-/** Old version of PowerPC, this should be removed shortly. */
-#define EM_PPC_OLD	17
-
-
 /** Cygnus M32R ELF backend.  Written in the absence of an ABI.  */
 #define EM_CYGNUS_M32R 0x9041U
 
-/** Alpha backend magic number.  Written in the absence of an ABI.  */
-#define EM_ALPHA	0x9026U
-
 /** D10V backend magic number.  Written in the absence of an ABI.  */
 #define EM_CYGNUS_D10V	0x7650U
 
-
-
 /** mn10200 and mn10300 backend magic numbers.
    Written in the absense of an ABI.  */
 #define EM_CYGNUS_MN10200	0xdeadL
@@ -202,7 +281,7 @@
 #define NT_FPREGSET	2		/**< Contains copy of fpregset struct */
 #define NT_PRPSINFO	3		/**< Contains copy of prpsinfo struct */
 
-/** Values of note segment descriptor types for object files.  
+/** Values of note segment descriptor types for object files.
    (Only for hppa right now.  Should this be moved elsewhere?)  */
 
 #define NT_VERSION	1		/**< Contains a version string.  */
@@ -211,9 +290,9 @@
     which contains the symbol binding and symbol type.  The STB_ and STT_
     defines identify the binding and type. */
 
-#define ELF_ST_BIND(val)		(((uint8_t)(val)) >> 4)
-#define ELF_ST_TYPE(val)		(((uint8_t)(val)) & 0xF)
-#define ELF_ST_INFO(bind,type)		((((uint8_t)(bind)) << 4) + ((((uint8_t)type)) & 0xF))
+#define ELF_ST_BIND(val)		(((tUInt8)(val)) >> 4)
+#define ELF_ST_TYPE(val)		(((tUInt8)(val)) & 0xF)
+#define ELF_ST_INFO(bind,type)		((((tUInt8)(bind)) << 4) + ((((tUInt8)type)) & 0xF))
 
 #define STN_UNDEF	0		/**< undefined symbol index */
 
@@ -352,158 +431,158 @@
 
 
 typedef struct {
-  uint8_t	e_ident[16];		/**< ELF "magic number" */
-  uint8_t	e_type[2];		/**< Identifies object file type */
-  uint8_t	e_machine[2];		/**< Specifies required architecture */
-  uint8_t	e_version[4];		/**< Identifies object file version */
-  uint8_t	e_entry[4];		/**< Entry point virtual address */
-  uint8_t	e_phoff[4];		/**< Program header table file offset */
-  uint8_t	e_shoff[4];		/**< Section header table file offset */
-  uint8_t	e_flags[4];		/**< Processor-specific flags */
-  uint8_t	e_ehsize[2];		/**< ELF header size in bytes */
-  uint8_t	e_phentsize[2];		/**< Program header table entry size */
-  uint8_t	e_phnum[2];		/**< Program header table entry count */
-  uint8_t	e_shentsize[2];		/**< Section header table entry size */
-  uint8_t	e_shnum[2];		/**< Section header table entry count */
-  uint8_t	e_shstrndx[2];		/**< Section header string table index */
+  tUInt8	e_ident[16];		/**< ELF "magic number" */
+  tUInt8	e_type[2];		/**< Identifies object file type */
+  tUInt8	e_machine[2];		/**< Specifies required architecture */
+  tUInt8	e_version[4];		/**< Identifies object file version */
+  tUInt8	e_entry[4];		/**< Entry point virtual address */
+  tUInt8	e_phoff[4];		/**< Program header table file offset */
+  tUInt8	e_shoff[4];		/**< Section header table file offset */
+  tUInt8	e_flags[4];		/**< Processor-specific flags */
+  tUInt8	e_ehsize[2];		/**< ELF header size in bytes */
+  tUInt8	e_phentsize[2];		/**< Program header table entry size */
+  tUInt8	e_phnum[2];		/**< Program header table entry count */
+  tUInt8	e_shentsize[2];		/**< Section header table entry size */
+  tUInt8	e_shnum[2];		/**< Section header table entry count */
+  tUInt8	e_shstrndx[2];		/**< Section header string table index */
 } Elf386_External_Ehdr;
 
 typedef struct {
-  uint8_t	e_ident[16];		/**< ELF "magic number" */
-  uint8_t	e_type[2];		/**< Identifies object file type */
-  uint8_t	e_machine[2];		/**< Specifies required architecture */
-  uint8_t	e_version[4];		/**< Identifies object file version */
-  uint8_t	e_entry[8];		/**< Entry point virtual address */
-  uint8_t	e_phoff[8];		/**< Program header table file offset */
-  uint8_t	e_shoff[8];		/**< Section header table file offset */
-  uint8_t	e_flags[4];		/**< Processor-specific flags */
-  uint8_t	e_ehsize[2];		/**< ELF header size in bytes */
-  uint8_t	e_phentsize[2];		/**< Program header table entry size */
-  uint8_t	e_phnum[2];		/**< Program header table entry count */
-  uint8_t	e_shentsize[2];		/**< Section header table entry size */
-  uint8_t	e_shnum[2];		/**< Section header table entry count */
-  uint8_t	e_shstrndx[2];		/**< Section header string table index */
+  tUInt8	e_ident[16];		/**< ELF "magic number" */
+  tUInt8	e_type[2];		/**< Identifies object file type */
+  tUInt8	e_machine[2];		/**< Specifies required architecture */
+  tUInt8	e_version[4];		/**< Identifies object file version */
+  tUInt8	e_entry[8];		/**< Entry point virtual address */
+  tUInt8	e_phoff[8];		/**< Program header table file offset */
+  tUInt8	e_shoff[8];		/**< Section header table file offset */
+  tUInt8	e_flags[4];		/**< Processor-specific flags */
+  tUInt8	e_ehsize[2];		/**< ELF header size in bytes */
+  tUInt8	e_phentsize[2];		/**< Program header table entry size */
+  tUInt8	e_phnum[2];		/**< Program header table entry count */
+  tUInt8	e_shentsize[2];		/**< Section header table entry size */
+  tUInt8	e_shnum[2];		/**< Section header table entry count */
+  tUInt8	e_shstrndx[2];		/**< Section header string table index */
 } Elf64_External_Ehdr;
 
 /** Program header */
 
 typedef struct {
-  uint8_t	p_type[4];		/**< Identifies program segment type */
-  uint8_t	p_offset[4];		/**< Segment file offset */
-  uint8_t	p_vaddr[4];		/**< Segment virtual address */
-  uint8_t	p_paddr[4];		/**< Segment physical address (ignored on SystemV) */
-  uint8_t	p_filesz[4];		/**< Segment size in file */
-  uint8_t	p_memsz[4];		/**< Segment size in memory */
-  uint8_t	p_flags[4];		/**< Segment flags */
-  uint8_t	p_align[4];		/**< Segment alignment, file & memory */
+  tUInt8	p_type[4];		/**< Identifies program segment type */
+  tUInt8	p_offset[4];		/**< Segment file offset */
+  tUInt8	p_vaddr[4];		/**< Segment virtual address */
+  tUInt8	p_paddr[4];		/**< Segment physical address (ignored on SystemV) */
+  tUInt8	p_filesz[4];		/**< Segment size in file */
+  tUInt8	p_memsz[4];		/**< Segment size in memory */
+  tUInt8	p_flags[4];		/**< Segment flags */
+  tUInt8	p_align[4];		/**< Segment alignment, file & memory */
 } Elf386_External_Phdr;
 
 typedef struct {
-  uint8_t	p_type[4];		/**< Identifies program segment type */
-  uint8_t	p_flags[4];		/**< Segment flags */
-  uint8_t	p_offset[8];		/**< Segment file offset */
-  uint8_t	p_vaddr[8];		/**< Segment virtual address */
-  uint8_t	p_paddr[8];		/**< Segment physical address (ignored on SystemV)*/
-  uint8_t	p_filesz[8];		/**< Segment size in file */
-  uint8_t	p_memsz[8];		/**< Segment size in memory */
-  uint8_t	p_align[8];		/**< Segment alignment, file & memory */
+  tUInt8	p_type[4];		/**< Identifies program segment type */
+  tUInt8	p_flags[4];		/**< Segment flags */
+  tUInt8	p_offset[8];		/**< Segment file offset */
+  tUInt8	p_vaddr[8];		/**< Segment virtual address */
+  tUInt8	p_paddr[8];		/**< Segment physical address (ignored on SystemV)*/
+  tUInt8	p_filesz[8];		/**< Segment size in file */
+  tUInt8	p_memsz[8];		/**< Segment size in memory */
+  tUInt8	p_align[8];		/**< Segment alignment, file & memory */
 } Elf64_External_Phdr;
 
 /** Section header */
 
 typedef struct {
-  uint8_t	sh_name[4];		/**< Section name, index in string tbl */
-  uint8_t	sh_type[4];		/**< Type of section */
-  uint8_t	sh_flags[4];		/**< Miscellaneous section attributes */
-  uint8_t	sh_addr[4];		/**< Section virtual addr at execution */
-  uint8_t	sh_offset[4];		/**< Section file offset */
-  uint8_t	sh_size[4];		/**< Size of section in bytes */
-  uint8_t	sh_link[4];		/**< Index of another section */
-  uint8_t	sh_info[4];		/**< Additional section information */
-  uint8_t	sh_addralign[4];	/**< Section alignment */
-  uint8_t	sh_entsize[4];		/**< Entry size if section holds table */
+  tUInt8	sh_name[4];		/**< Section name, index in string tbl */
+  tUInt8	sh_type[4];		/**< Type of section */
+  tUInt8	sh_flags[4];		/**< Miscellaneous section attributes */
+  tUInt8	sh_addr[4];		/**< Section virtual addr at execution */
+  tUInt8	sh_offset[4];		/**< Section file offset */
+  tUInt8	sh_size[4];		/**< Size of section in bytes */
+  tUInt8	sh_link[4];		/**< Index of another section */
+  tUInt8	sh_info[4];		/**< Additional section information */
+  tUInt8	sh_addralign[4];	/**< Section alignment */
+  tUInt8	sh_entsize[4];		/**< Entry size if section holds table */
 } Elf386_External_Shdr;
 
 typedef struct {
-  uint8_t	sh_name[4];		/**< Section name, index in string tbl */
-  uint8_t	sh_type[4];		/**< Type of section */
-  uint8_t	sh_flags[8];		/**< Miscellaneous section attributes */
-  uint8_t	sh_addr[8];		/**< Section virtual addr at execution */
-  uint8_t	sh_offset[8];		/**< Section file offset */
-  uint8_t	sh_size[8];		/**< Size of section in bytes */
-  uint8_t	sh_link[4];		/**< Index of another section */
-  uint8_t	sh_info[4];		/**< Additional section information */
-  uint8_t	sh_addralign[8];	/**< Section alignment */
-  uint8_t	sh_entsize[8];		/**< Entry size if section holds table */
+  tUInt8	sh_name[4];		/**< Section name, index in string tbl */
+  tUInt8	sh_type[4];		/**< Type of section */
+  tUInt8	sh_flags[8];		/**< Miscellaneous section attributes */
+  tUInt8	sh_addr[8];		/**< Section virtual addr at execution */
+  tUInt8	sh_offset[8];		/**< Section file offset */
+  tUInt8	sh_size[8];		/**< Size of section in bytes */
+  tUInt8	sh_link[4];		/**< Index of another section */
+  tUInt8	sh_info[4];		/**< Additional section information */
+  tUInt8	sh_addralign[8];	/**< Section alignment */
+  tUInt8	sh_entsize[8];		/**< Entry size if section holds table */
 } Elf64_External_Shdr;
 
 /** Symbol table entry */
 
 typedef struct {
-  uint8_t	st_name[4];		/**< Symbol name, index in string tbl */
-  uint8_t	st_value[4];		/**< Value of the symbol */
-  uint8_t	st_size[4];		/**< Associated symbol size */
-  uint8_t	st_info[1];		/**< Type and binding attributes */
-  uint8_t	st_other[1];		/**< No defined meaning, 0 */
-  uint8_t	st_shndx[2];		/**< Associated section index */
+  tUInt8	st_name[4];		/**< Symbol name, index in string tbl */
+  tUInt8	st_value[4];		/**< Value of the symbol */
+  tUInt8	st_size[4];		/**< Associated symbol size */
+  tUInt8	st_info[1];		/**< Type and binding attributes */
+  tUInt8	st_other[1];		/**< No defined meaning, 0 */
+  tUInt8	st_shndx[2];		/**< Associated section index */
 } Elf386_External_Sym;
 
 typedef struct {
-  uint8_t	st_name[4];		/**< Symbol name, index in string tbl */
-  uint8_t	st_info[1];		/**< Type and binding attributes */
-  uint8_t	st_other[1];		/**< No defined meaning, 0 */
-  uint8_t	st_shndx[2];		/**< Associated section index */
-  uint8_t	st_value[8];		/**< Value of the symbol */
-  uint8_t	st_size[8];		/**< Associated symbol size */
+  tUInt8	st_name[4];		/**< Symbol name, index in string tbl */
+  tUInt8	st_info[1];		/**< Type and binding attributes */
+  tUInt8	st_other[1];		/**< No defined meaning, 0 */
+  tUInt8	st_shndx[2];		/**< Associated section index */
+  tUInt8	st_value[8];		/**< Value of the symbol */
+  tUInt8	st_size[8];		/**< Associated symbol size */
 } Elf64_External_Sym;
 
 /** Note segments */
 
 typedef struct {
-  uint8_t	namesz[4];		/**< Size of entry's owner string */
-  uint8_t	descsz[4];		/**< Size of the note descriptor */
-  uint8_t	type[4];		/**< Interpretation of the descriptor */
-  int8_t	name[1];		/**< Start of the name+desc data */
+  tUInt8	namesz[4];		/**< Size of entry's owner string */
+  tUInt8	descsz[4];		/**< Size of the note descriptor */
+  tUInt8	type[4];		/**< Interpretation of the descriptor */
+  tInt8	name[1];		/**< Start of the name+desc data */
 } Elf_External_Note;
 
 /** Relocation Entries */
 typedef struct {
-  uint8_t r_offset[4];	/**< Location at which to apply the action */
-  uint8_t	r_info[4];	/**< index and type of relocation */
+  tUInt8	r_offset[4];	/**< Location at which to apply the action */
+  tUInt8	r_info[4];	/**< index and type of relocation */
 } Elf386_External_Rel;
 
 typedef struct {
-  uint8_t r_offset[4];	/**< Location at which to apply the action */
-  uint8_t	r_info[4];	/**< index and type of relocation */
-  uint8_t	r_addend[4];	/**< Constant addend used to compute value */
+  tUInt8	r_offset[4];	/**< Location at which to apply the action */
+  tUInt8	r_info[4];	/**< index and type of relocation */
+  tUInt8	r_addend[4];	/**< Constant addend used to compute value */
 } Elf386_External_Rela;
 
 typedef struct {
-  uint8_t r_offset[8];	/**< Location at which to apply the action */
-  uint8_t	r_info[8];	/**< index and type of relocation */
+  tUInt8	r_offset[8];	/**< Location at which to apply the action */
+  tUInt8	r_info[8];	/**< index and type of relocation */
 } Elf64_External_Rel;
 
 typedef struct {
-  uint8_t r_offset[8];	/**< Location at which to apply the action */
-  uint8_t	r_info[8];	/**< index and type of relocation */
-  uint8_t	r_addend[8];	/**< Constant addend used to compute value */
+  tUInt8	r_offset[8];	/**< Location at which to apply the action */
+  tUInt8	r_info[8];	/**< index and type of relocation */
+  tUInt8	r_addend[8];	/**< Constant addend used to compute value */
 } Elf64_External_Rela;
 
 /** dynamic section structure */
 
 typedef struct {
-  uint8_t	d_tag[4];		/**< entry tag value */
+  tUInt8	d_tag[4];		/**< entry tag value */
   union {
-    uint8_t	d_val[4];
-    uint8_t	d_ptr[4];
+    tUInt8	d_val[4];
+    tUInt8	d_ptr[4];
   } d_un;
 } Elf386_External_Dyn;
 
 typedef struct {
-  uint8_t	d_tag[8];		/**< entry tag value */
+  tUInt8	d_tag[8];		/**< entry tag value */
   union {
-    uint8_t	d_val[8];
-    uint8_t	d_ptr[8];
+    tUInt8	d_val[8];
+    tUInt8	d_ptr[8];
   } d_un;
 } Elf64_External_Dyn;
 
@@ -514,47 +593,47 @@ typedef struct {
 /** This structure appears in a SHT_GNU_verdef section.  */
 
 typedef struct {
-  uint8_t		vd_version[2];
-  uint8_t		vd_flags[2];
-  uint8_t		vd_ndx[2];
-  uint8_t		vd_cnt[2];
-  uint8_t		vd_hash[4];
-  uint8_t		vd_aux[4];
-  uint8_t		vd_next[4];
+  tUInt8		vd_version[2];
+  tUInt8		vd_flags[2];
+  tUInt8		vd_ndx[2];
+  tUInt8		vd_cnt[2];
+  tUInt8		vd_hash[4];
+  tUInt8		vd_aux[4];
+  tUInt8		vd_next[4];
 } Elf_External_Verdef;
 
 /** This structure appears in a SHT_GNU_verdef section.  */
 
 typedef struct {
-  uint8_t		vda_name[4];
-  uint8_t		vda_next[4];
+  tUInt8		vda_name[4];
+  tUInt8		vda_next[4];
 } Elf_External_Verdaux;
 
 /** This structure appears in a SHT_GNU_verneed section.  */
 
 typedef struct {
-  uint8_t		vn_version[2];
-  uint8_t		vn_cnt[2];
-  uint8_t		vn_file[4];
-  uint8_t		vn_aux[4];
-  uint8_t		vn_next[4];
+  tUInt8		vn_version[2];
+  tUInt8		vn_cnt[2];
+  tUInt8		vn_file[4];
+  tUInt8		vn_aux[4];
+  tUInt8		vn_next[4];
 } Elf_External_Verneed;
 
 /** This structure appears in a SHT_GNU_verneed section.  */
 
 typedef struct {
-  uint8_t		vna_hash[4];
-  uint8_t		vna_flags[2];
-  uint8_t		vna_other[2];
-  uint8_t		vna_name[4];
-  uint8_t		vna_next[4];
+  tUInt8		vna_hash[4];
+  tUInt8		vna_flags[2];
+  tUInt8		vna_other[2];
+  tUInt8		vna_name[4];
+  tUInt8		vna_next[4];
 } Elf_External_Vernaux;
 
 /** This structure appears in a SHT_GNU_versym section.  This is not a
    standard ELF structure; ELF just uses Elf386_Half.  */
 
 typedef struct {
-  uint8_t		vs_vers[2];
+  tUInt8		vs_vers[2];
 } Elf_External_Versym;
 
 /** Auxiliary vector.  */
@@ -568,10 +647,10 @@ typedef struct {
 
 typedef struct
 {
-  int32_t a_type;		/**< Entry type */
+  tInt32 a_type;		/**< Entry type */
   union
     {
-      int32_t a_val;		/**< Integer value */
+      tInt32 a_val;		/**< Integer value */
       void *a_ptr;		/**< Pointer value */
       void (*a_fcn) (void);	/**< Function pointer value */
     } a_un;
@@ -579,10 +658,10 @@ typedef struct
 
 typedef struct
 {
-  int32_t a_type;		/**< Entry type */
+  tInt32 a_type;		/**< Entry type */
   union
     {
-      int32_t a_val;		/**< Integer value */
+      tInt32 a_val;		/**< Integer value */
       void *a_ptr;		/**< Pointer value */
       void (*a_fcn) (void);	/**< Function pointer value */
     } a_un;
@@ -733,13 +812,13 @@ typedef union
 {
   struct
     {
-      uint8_t gt_current_g_value[4];	/**< -G value used for compilation */
-      uint8_t gt_unused[4];		/**< Not used */
+      tUInt8 gt_current_g_value[4];	/**< -G value used for compilation */
+      tUInt8 gt_unused[4];		/**< Not used */
     } gt_header;        	   	/**< First entry in section */
   struct
     {
-      uint8_t gt_g_value[4];		/**< If this value were used for -G */
-      uint8_t gt_bytes[4];		/**< This many bytes would be used */
+      tUInt8 gt_g_value[4];		/**< If this value were used for -G */
+      tUInt8 gt_bytes[4];		/**< This many bytes would be used */
     } gt_entry;				/**< Subsequent entries in section */
 } Elf386_gptab;
 
@@ -747,9 +826,9 @@ typedef union
 
 typedef struct
 {
-  uint8_t	ri_gprmask[4];		/**< General registers used */
-  uint8_t	ri_cprmask[4][4];	/**< Coprocessor registers used */
-  uint8_t	ri_gp_value[4];		/**< $gp register value */
+  tUInt8	ri_gprmask[4];		/**< General registers used */
+  tUInt8	ri_cprmask[4][4];	/**< Coprocessor registers used */
+  tUInt8	ri_gp_value[4];		/**< $gp register value */
 } Elf386_RegInfo;
 
 /** MIPS relocs.  */
@@ -804,11 +883,11 @@ typedef struct
 
 typedef struct
 {
-  uint8_t	l_name[4];		/**< Name (string table index) */
-  uint8_t	l_time_stamp[4];	/**< Timestamp */
-  uint8_t	l_checksum[4];		/**< Checksum */
-  uint8_t	l_version[4];		/**< Interface version */
-  uint8_t	l_flags[4];		/**< Flags */
+  tUInt8	l_name[4];		/**< Name (string table index) */
+  tUInt8	l_time_stamp[4];	/**< Timestamp */
+  tUInt8	l_checksum[4];		/**< Checksum */
+  tUInt8	l_version[4];		/**< Interface version */
+  tUInt8	l_flags[4];		/**< Flags */
 } Elf386_Lib;
 
 /** Legal values for l_flags.  */
