@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/ne.c
  * @brief       This file contains implementation of NE (New Executable) file
@@ -46,6 +46,7 @@ static unsigned long __NEAR__ __FASTCALL__ CalcEntryPointNE( unsigned,unsigned )
 static void __FASTCALL__ ne_ReadPubNameList(BGLOBAL handle,void (__FASTCALL__ *mem_out)(const char *));
 static tBool  __NEAR__ __FASTCALL__ FindPubName(char *buff,unsigned cb_buff,unsigned long pa);
 static void __FASTCALL__ rd_ImpName(char *buff,int blen,unsigned idx,tBool useasoff);
+static unsigned long __FASTCALL__ nePA2VA(unsigned long pa);
 
 static tBool __FASTCALL__ neLowMemFunc( unsigned long need_mem )
 {
@@ -190,16 +191,18 @@ static void __NEAR__ PaintNewHeaderNE_2( void )
            ,GetBool(ne.neFlagsOther & 0x80));
   if(ne.neOperatingSystem == 2)
   {
-    char high,low;
+	/* Correction by Olivier Mengu\u00e9*/
+    unsigned char high,low;
     high = ne.neWindowsVersion >> 8;
-    low  = ne.neWindowsVersion;
+    low  = ne.neWindowsVersion & 0xFF;
+	/* End of correction */
     twSetColorAttr(dialog_cset.addinfo);
     twPrintF("Offset of Fast Load Area       = %04hXH"
              ,ne.neOffsetFastLoadArea); twClrEOL();
     twPrintF("\nLength of Fast Load Area       = %hu"
              ,ne.neLengthFastLoadArea); twClrEOL();
     twPrintF("\nWindows version                = %02hu.%02hu"
-             ,(int)high,(int)low); twClrEOL();
+             ,(unsigned int)high,(unsigned int)low); twClrEOL();
     twPrintF("\n");
   }
   twSetColorAttr(dialog_cset.entry);
@@ -338,7 +341,7 @@ static void __FASTCALL__ EntPaintNE(TWindow * win,const void ** names,unsigned s
 static tBool __FASTCALL__ __ReadModRefNamesNE(BGLOBAL handle,memArray * obj)
 {
  unsigned i;
- unsigned short offTable;
+ tUIntFast16 offTable;
  bioSeek(handle,ne.neOffsetModuleReferenceTable + headshift,SEEKF_START);
  for(i = 0;i < ne.neModuleReferenceTableCount;i++)
  {
@@ -348,7 +351,7 @@ static tBool __FASTCALL__ __ReadModRefNamesNE(BGLOBAL handle,memArray * obj)
    char stmp[256];
    offTable = bioReadWord(handle);
    fp = bioTell(handle);
-   NameOff = (long)headshift + (long)offTable + ne.neOffsetImportTable;
+   NameOff = (long)headshift + offTable + ne.neOffsetImportTable;
    bioSeek(handle,NameOff,SEEKF_START);
    length = bioReadByte(handle);
    if(IsKbdTerminate() || bioEOF(handle)) break;
@@ -426,8 +429,7 @@ static tBool __FASTCALL__ __ReadProcListNE(BGLOBAL handle,memArray * obj,int mod
     if(tsd.sdLength && tsd.sdOffset && tsd.sdFlags & 0x0100)
     {
       unsigned long spos;
-      unsigned j;
-      unsigned short nrelocs;
+      tUIntFast16 j,nrelocs;
       RELOC_NE rne;
       spos = bioTell(handle);
       bioSeek(handle,((long)(tsd.sdOffset) << ne.neLogicalSectorShiftCount) + tsd.sdLength,SEEKF_START);
@@ -488,7 +490,7 @@ static long RNRprevshift = 0;
 static unsigned __FASTCALL__ RNameReadFull(BGLOBAL handle,char * names,unsigned nindex,unsigned long offset)
 {
  unsigned char length;
- unsigned short Ordinal;
+ tUIntFast16 Ordinal;
  unsigned i;
  if(RNRprevind == (nindex - 1) && RNRprevshift)  bioSeek(handle,RNRprevshift,SEEKF_START);
  else
@@ -522,35 +524,17 @@ static unsigned __FASTCALL__ NResNameReadFull(BGLOBAL handle,char * names,unsign
 tBool __FASTCALL__ RNamesReadItems(BGLOBAL handle,memArray * obj,unsigned nnames,unsigned long offset)
 {
  unsigned char length;
- unsigned short Ordinal;
- unsigned i,j;
- char buffer1[71];
+ unsigned Ordinal;
+ unsigned i;
+ char stmp[300]; /* max255 + @ordinal */
  bioSeek(handle,offset,SEEKF_START);
  for(i = 0;i < nnames;i++)
  {
-   char stmp[83];
    length = bioReadByte(handle);
    if(IsKbdTerminate() || bioEOF(handle)) break;
-   memset(buffer1,0,sizeof(buffer1));
-   if(length > 66)
-   {
-     bioReadBuffer(handle,(void *)buffer1,66);
-     buffer1[66] = TWC_RT_ARROW;
-     buffer1[67] = TWC_RT_ARROW;
-     buffer1[68] = TWC_RT_ARROW;
-     buffer1[69] = 0;
-     bioSeek(handle,length - 66,SEEKF_CUR);
-   }
-   else
-   {
-     bioReadBuffer(handle,buffer1,length);
-     buffer1[length] = 0;
-   }
+   bioReadBuffer(handle,stmp,length);
    Ordinal = bioReadWord(handle);
-   strcpy(stmp,buffer1);
-   j = strlen(stmp);
-   if(j < 69) { memset(&stmp[j],' ',69-j); stmp[69] = 0; }
-   sprintf(&stmp[strlen(stmp)]," %u",Ordinal);
+   sprintf(&stmp[length],"%c%-5u",LB_ORD_DELIMITER, Ordinal);
    if(!ma_AddString(obj,stmp,True)) break;
  }
  return True;
@@ -688,7 +672,7 @@ static unsigned long __NEAR__ __FASTCALL__ CalcEntryNE(unsigned ord,tBool dispms
   ENTRY entr;
   SEGDEF segd;
   int segnum;
-  if(!ReadEntryNE(&entr,ord)) { if(dispmsg) ErrMessageBox("No such entry in the Table",NULL); return 0L; }
+  if(!ReadEntryNE(&entr,ord)) { if(dispmsg) ErrMessageBox(NOT_ENTRY,NULL); return 0L; }
   if(entr.eFixed == 0xFE)
   {
     char outs[100];
@@ -848,7 +832,7 @@ static char * __NEAR__ __FASTCALL__ GetResourceIDNE(BGLOBAL handle,unsigned rid,
 static tBool __NEAR__ __FASTCALL__ __ReadResourceGroupNE(BGLOBAL handle,memArray *obj,unsigned nitems,long * addr)
 {
  unsigned i,j;
- unsigned short rcAlign,rTypeID,rcount;
+ tUIntFast16 rcAlign,rTypeID,rcount;
  unsigned long BegResTab;
  char buff[81];
  BegResTab = bioTell(handle);
@@ -892,8 +876,7 @@ static tBool __NEAR__ __FASTCALL__ __ReadResourceGroupNE(BGLOBAL handle,memArray
 
 static unsigned int __NEAR__ __FASTCALL__ GetResourceGroupCountNE(BGLOBAL handle)
 {
- unsigned short rcount;
- unsigned short rTypeID;
+ tUIntFast16 rcount, rTypeID;
  int count = 0;
  unsigned long pos;
  if(ne.neOffsetResourceTable == ne.neOffsetResidentNameTable) return 0;
@@ -1007,10 +990,7 @@ static tCompare __FASTCALL__ compare_chains(const void __HUGE__ *v1,const void _
   const NERefChain __HUGE__ * c1,__HUGE__ * c2;
   c1 = (const NERefChain __HUGE__ *)v1;
   c2 = (const NERefChain __HUGE__ *)v2;
-  if(c1->offset < c2->offset) return -1;
-  else
-    if(c1->offset > c2->offset) return 1;
-    else                        return 0;
+  return __CmpLong__(c1->offset,c2->offset);
 }
 
 
@@ -1021,7 +1001,7 @@ static void __NEAR__ __FASTCALL__ BuildNERefChain(unsigned long segoff,unsigned
   TWindow * w,*usd;
   usd = twUsedWin();
   if(!(CurrNEChain = la_Build(0,sizeof(NERefChain),MemOutBox))) return;
-  w = CrtDlgWndnls(" System is busy ",49,1);
+  w = CrtDlgWndnls(SYSTEM_BUSY,49,1);
   twUseWin(w);
   twGotoXY(1,1);
   twPrintF(" Building reference chains for segment #%u",CurrChainSegment);
@@ -1169,15 +1149,17 @@ static void __FASTCALL__ rd_ImpName(char *buff,int blen,unsigned idx,tBool useas
   rdImpNameNELX(buff,blen,idx,useasoff,headshift + ne.neOffsetImportTable);
 }
 
-static int __NEAR__ __FASTCALL__ BuildReferStrNE(char *str,RELOC_NE *rne,int flags,unsigned long ulShift)
+static unsigned long __NEAR__ __FASTCALL__ BuildReferStrNE(char *str,RELOC_NE *rne,int flags,unsigned long ulShift)
 {
   char buff[256];
   const char *pref;
-  int retrf;
+  unsigned long retrf;
   char reflen;
+  tBool need_virt;
   reflen = 0;
   pref = "";
-  retrf = RAPREF_PROCEEDCMT;
+  retrf = RAPREF_NONE;
+  need_virt = (flags & APREF_SAVE_VIRT);
   switch(rne->AddrType)
   {
       case 0: reflen = 1; pref = "(b) "; break;
@@ -1185,13 +1167,13 @@ static int __NEAR__ __FASTCALL__ BuildReferStrNE(char *str,RELOC_NE *rne,int fla
       case 3: reflen = 4; pref = "seg:off16 "; break;
       case 5: reflen = 2; pref = "off16 "; break;
       case 11: reflen = 6; pref = "seg:off32 "; break;
-      case 13: reflen = 4; pref = "off32 "; retrf = RAPREF_PROCEEDCMT; break;
+      case 13: reflen = 4; pref = "off32 "; break;
       default: break;
   }
   if(flags & APREF_USE_TYPE) strcat(str,pref);
   if((rne->Type & 3) == 1 || (rne->Type & 3) == 2) /** imported type */
   {
-    retrf = RAPREF_DONE; 
+    retrf = RAPREF_DONE;
     rd_ImpName(buff,sizeof(buff),rne->idx,0);
     sprintf(&str[strlen(str)],"<%s>.",buff);
     if((rne->Type & 3) == 1) sprintf(&str[strlen(str)],"@%hu",rne->ordinal);
@@ -1210,7 +1192,11 @@ static int __NEAR__ __FASTCALL__ BuildReferStrNE(char *str,RELOC_NE *rne,int fla
        ea = CalcEntryNE(rne->ordinal,False);
        if(FindPubName(buff,sizeof(buff),ea))
           sprintf(&str[strlen(str)],"%s",buff);
-       else sprintf(&str[strlen(str)],"(*this).@%hu",rne->ordinal);
+       else 
+       {
+         retrf = ea;
+         sprintf(&str[strlen(str)],"(*this).@%hu",rne->ordinal);
+       }
        if(!DumpMode && !EditMode && !(flags & APREF_USE_TYPE)) GidAddGoAddress(str,ea);
      }
      else
@@ -1218,10 +1204,13 @@ static int __NEAR__ __FASTCALL__ BuildReferStrNE(char *str,RELOC_NE *rne,int fla
        unsigned long ep;
        ep = CalcEntryPointNE(rne->idx,rne->ordinal);
        if(FindPubName(buff,sizeof(buff),ep))
-       {
          sprintf(&str[strlen(str)],"%s",buff);
+       else
+       {
+         if(need_virt) sprintf(&str[strlen(str)],".%08lX",nePA2VA(ep));
+         else sprintf(&str[strlen(str)],"(*this).seg<#%hu>:%sH",rne->idx,Get4Digit(rne->ordinal));
+         retrf = ep;
        }
-       else sprintf(&str[strlen(str)],"(*this).seg<#%hu>:%sH",rne->idx,Get4Digit(rne->ordinal));
        if(!DumpMode && !EditMode && !(flags & APREF_USE_TYPE)) GidAddGoAddress(str,ep);
      }
    }
@@ -1243,11 +1232,12 @@ static int __NEAR__ __FASTCALL__ BuildReferStrNE(char *str,RELOC_NE *rne,int fla
    return retrf;
 }
 
-static int __FASTCALL__ AppendNERef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
+static unsigned long __FASTCALL__ AppendNERef(char *str,unsigned long ulShift,int flags,int codelen,unsigned long r_sh)
 {
     unsigned i;
     unsigned long segpos,slength;
     char buff[256];
+    if(flags & APREF_TRY_PIC) return RAPREF_NONE;
     if(ulShift >= CurrSegmentStart && ulShift <= CurrSegmentStart + CurrSegmentLength)
     {
        i = CurrChainSegment - 1;
@@ -1414,9 +1404,9 @@ static unsigned long __FASTCALL__ NEHelp( void )
 static unsigned long __FASTCALL__ neVA2PA(unsigned long va)
 {
   SEGDEF nesd;
-  unsigned short seg,off;
-  seg = (va >> 16) & 0xFFFF;
-  off = (unsigned short)(va & 0xFFFF);
+  tUIntFast16 seg,off;
+  seg = (va >> 16) & 0xFFFFU;
+  off = va & 0xFFFFU;
   if(!ReadSegDefNE(&nesd,seg)) return 0L;
   return nesd.sdOffset ? (((unsigned long)nesd.sdOffset)<<ne.neLogicalSectorShiftCount) + off : 0;
 }
@@ -1427,19 +1417,22 @@ static unsigned long __FASTCALL__ nePA2VA(unsigned long pa)
   unsigned long currseg_st,nextseg_st;
   for(i = 0;i < segcount-1;i++)
   {
-    SEGDEF nesd_c,nesd_n;
+    SEGDEF nesd_c/*,nesd_n*/;
     if(!ReadSegDefNE(&nesd_c,i+1)) return 0L;
     currseg_st = (((unsigned long)nesd_c.sdOffset)<<ne.neLogicalSectorShiftCount);
     if(!currseg_st) continue;
-    if(!ReadSegDefNE(&nesd_n,i+2)) goto it_seg;
-    nextseg_st = (((unsigned long)nesd_n.sdOffset)<<ne.neLogicalSectorShiftCount);
+/*    if(!ReadSegDefNE(&nesd_n,i+2)) goto it_seg;*/
+/*    nextseg_st = (((unsigned long)nesd_n.sdOffset)<<ne.neLogicalSectorShiftCount);*/
+    nextseg_st = currseg_st + nesd_c.sdLength;
     if(pa >= currseg_st && pa < nextseg_st)
     {
-      it_seg:
+/*      it_seg:*/
       return ((unsigned long)(i+1) << 16) + (unsigned)(pa - currseg_st);
     }
+/*
     if(i == segcount-2 && pa >= nextseg_st)
       return ((unsigned long)(i+2) << 16) + (unsigned)(pa - nextseg_st);
+*/
   }
   return 0L;
 }
@@ -1453,7 +1446,7 @@ static unsigned long __FASTCALL__ neGetPubSym(char *str,unsigned cb_str,unsigned
 }
 
 static unsigned __FASTCALL__ neGetObjAttr(unsigned long pa,char *name,unsigned cb_name,
-                      unsigned long *start,unsigned long *end,int *class,int *bitness)
+                      unsigned long *start,unsigned long *end,int *_class,int *bitness)
 {
   unsigned long currseg_st;
   unsigned i,segcount = ne.neSegmentTableCount,ret;
@@ -1462,7 +1455,7 @@ static unsigned __FASTCALL__ neGetObjAttr(unsigned long pa,char *name,unsigned c
   UNUSED(cb_name);
   *start = 0;
   *end = bmGetFLength();
-  *class = OC_NOOBJECT;
+  *_class = OC_NOOBJECT;
   *bitness = DAB_USE16;
   name[0] = 0;
   ret = 0;
@@ -1488,7 +1481,7 @@ static unsigned __FASTCALL__ neGetObjAttr(unsigned long pa,char *name,unsigned c
     {
       *start = currseg_st;
       *end = *start + nesd_c.sdLength;
-      *class = nesd_c.sdFlags & 0x01 ? OC_DATA : OC_CODE;
+      *_class = nesd_c.sdFlags & 0x01 ? OC_DATA : OC_CODE;
       *bitness = nesd_c.sdFlags & 0x2000 ? DAB_USE32 : DAB_USE16;
       ret = i+1;
       found = True;
@@ -1510,11 +1503,11 @@ static int __FASTCALL__ bitnessNE(unsigned long pa)
 {
   static unsigned long st = 0,end = 0;
   char name[1];
-  int class;
+  int _class;
   static int bitness;
   if(!(pa >= st && pa < end))
   {
-    neGetObjAttr(pa,name,sizeof(name),&st,&end,&class,&bitness);
+    neGetObjAttr(pa,name,sizeof(name),&st,&end,&_class,&bitness);
   }
   return bitness;
 }
@@ -1524,7 +1517,7 @@ static tBool __FASTCALL__ neAddressResolv(char *addr,unsigned long cfpos)
  /* Since this function is used in references resolving of disassembler
     it must be seriously optimized for speed. */
   tBool bret = True;
-  uint32_t res;
+  tUInt32 res;
   if(cfpos >= headshift && cfpos < headshift + sizeof(NEHEADER))
   {
      strcpy(addr,"NEH :");
@@ -1544,7 +1537,7 @@ static tBool __FASTCALL__ neAddressResolv(char *addr,unsigned long cfpos)
     strcpy(addr,"NEET:");
     strcpy(&addr[5],Get4Digit(cfpos - headshift - ne.neOffsetEntryTable));
   }
-  else 
+  else
     if((res=nePA2VA(cfpos))!=0)
     {
       addr[0] = '.';
