@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_auto
  * @file        plugins/bin/rdoff2.h
  * @brief       This file contains RDOFF v2 file definitions.
@@ -28,9 +28,9 @@
 /** RDOFF v2 fixed header */
 typedef struct tag_rdoff2_Header
 {
-  uint8_t  id[6]; /**< RDOFF2 or RDOFF\02 */
-  uint32_t image_len;
-  uint32_t header_len;
+  tUInt8  id[6]; /**< RDOFF2 or RDOFF\02 */
+  tUInt32 image_len;
+  tUInt32 header_len;
 }rdoff2_Header;
 
 
