@@ -1,4 +1,4 @@
-/** 
+/**
  * @namespace   biew_plugins_II
  * @file        plugins/disasm/ix86/ix86_fun.c
  * @brief       This file contains implementation common function and utility
@@ -41,14 +41,32 @@ char *ix86_Katmai_buff;
 #define getSREG(sreg) (ix86_SegRegs[(unsigned char)sreg])
 #define getTile(DisP) __getTile(DisP,DisP->RealCmd[0] & 0x01,DisP->RealCmd[0] & 0x02)
 
-static const char * __FASTCALL__ ix86_getREG(unsigned char reg,tBool w)
+#ifdef IX86_64
+#define REX_reg(rex,reg) (reg|((rex&1)<<3))
+#endif
+
+static const char * __FASTCALL__ k86_getREG(unsigned char reg,tBool w,tBool rex, tBool use64)
 {
+#ifdef IX86_64
+ if(UseXMMXSet) return x86_Bitness==DAB_USE64?k86_XMMXRegs[REX_reg(rex,reg)]:ix86_XMMXRegs[reg];
+ else
+   if(UseMMXSet)  return ix86_MMXRegs[reg];
+   else
+     if(!w) return (x86_Bitness==DAB_USE64&&has_REX)?k86_ByteRegs[REX_reg(rex,reg)]:ix86_ByteRegs[reg];
+     else 
+     if(x86_Bitness==DAB_USE64)
+      return use64?k86_QWordRegs[REX_reg(rex,reg)]:Use32Data?k86_DWordRegs[REX_reg(rex,reg)]:k86_WordRegs[REX_reg(rex,reg)];
+     else return Use32Data ? ix86_DWordRegs[reg] : ix86_WordRegs[reg];
+#else
  if(UseXMMXSet) return ix86_XMMXRegs[reg];
  else
    if(UseMMXSet)  return ix86_MMXRegs[reg];
    else
      if(!w) return ix86_ByteRegs[reg];
      else   return Use32Data ? ix86_DWordRegs[reg] : ix86_WordRegs[reg];
+ rex=rex;
+ use64 = use64;
+#endif
 }
 
 static char * __NEAR__ __FASTCALL__ GetDigitsApp(unsigned char loc_off,
@@ -68,11 +86,15 @@ static char * __NEAR__ __FASTCALL__ GetDigitsApp(unsigned char loc_off,
 #define Get4SignDigApp(loc_off,DisP) GetDigitsApp(loc_off,DisP,2,DISARG_SHORT)
 #define Get8DigitApp(loc_off,DisP)   GetDigitsApp(loc_off,DisP,4,DISARG_DWORD)
 #define Get8SignDigApp(loc_off,DisP) GetDigitsApp(loc_off,DisP,4,DISARG_LONG)
-
+#ifdef IX86_64
+#define Get16DigitApp(loc_off,DisP)   GetDigitsApp(loc_off,DisP,8,DISARG_QWORD)
+#define Get16SignDigApp(loc_off,DisP) GetDigitsApp(loc_off,DisP,8,DISARG_LLONG)
+#endif
 static char * __NEAR__ __FASTCALL__ Get2SquareDig(unsigned char loc_off,ix86Param *DisP,tBool as_sign)
 {
   char *ptr = ix86_apistr;
   char *rets;
+  *ptr = 0;
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
   {
     rets = GetDigitsApp(loc_off,DisP,1,as_sign ? DISARG_CHAR : DISARG_BYTE);
@@ -87,6 +109,7 @@ static char * __NEAR__ __FASTCALL__ Get4SquareDig(unsigned char loc_off,ix86Para
   char *ptr = ix86_apistr;
   char *rets;
   unsigned type;
+  *ptr = 0;
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
   {
     type = as_sign ? DISARG_SHORT : DISARG_WORD;
@@ -103,6 +126,7 @@ static char * __NEAR__ __FASTCALL__ Get8SquareDig(unsigned char loc_off,ix86Para
   char *ptr = ix86_apistr;
   char *rets;
   unsigned type;
+  *ptr = 0;
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
   {
     type = as_sign ? DISARG_LONG : DISARG_DWORD;
@@ -114,6 +138,25 @@ static char * __NEAR__ __FASTCALL__ Get8SquareDig(unsigned char loc_off,ix86Para
   return ix86_apistr;
 }
 
+#ifdef IX86_64
+static char * __NEAR__ __FASTCALL__ Get16SquareDig(unsigned char loc_off,ix86Param *DisP,tBool as_sign,tBool is_disponly)
+{
+  char *ptr = ix86_apistr;
+  char *rets;
+  unsigned type;
+  *ptr = 0;
+  if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
+  {
+    type = as_sign ? DISARG_LLONG : DISARG_QWORD;
+    type |= is_disponly ? DISARG_DISP : DISARG_IDXDISP;
+    rets = GetDigitsApp(loc_off,DisP,8,type);
+    if(!(rets[0] == '+' || rets[0] == '-')) *ptr++ = '+';
+    strcpy(ptr,rets);
+  }
+  return ix86_apistr;
+}
+#endif
+
 void __FASTCALL__ ix86_ArgES(char *str,ix86Param *param)
 {
   UNUSED(param);
@@ -188,12 +231,31 @@ void __FASTCALL__ ix86_ArgDWord(char *str,ix86Param *DisP)
   DisP->codelen+=4;
 }
 
+static void __FASTCALL__ ix86_ArgQWord(char *str,ix86Param *DisP)
+{
+  if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
+    disAppendDigits(str,DisP->CodeAddress + DisP->codelen,
+                 APREF_USE_TYPE,8,&DisP->RealCmd[DisP->codelen],DISARG_QWORD | DISARG_IMM);
+  DisP->codelen+=8;
+}
+
 static char * __FASTCALL__ ix86_GetDigitTile(ix86Param* DisP,char wrd,char sgn,unsigned char loc_off)
 {
   int cl,type;
-  cl = wrd ? ( Use32Data ? 4 : 2 ) : 1;
+  int do_64;
+  do_64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64 && wrd == -1) /* special case for */
+    do_64 = 1;
+#endif
+  cl = do_64 ? 8 : wrd ? ( Use32Data ? 4 : 2 ) : 1;
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
   {
+#ifdef IX86_64
+    if(do_64)
+      type = sgn ? DISARG_LLONG : DISARG_QWORD;
+    else
+#endif
     type = sgn ? wrd ? ( Use32Data ? DISARG_LONG : DISARG_SHORT ) : DISARG_CHAR:
                  wrd ? ( Use32Data ? DISARG_DWORD : DISARG_WORD ) : DISARG_BYTE;
     type |= DISARG_IMM;
@@ -215,8 +277,7 @@ void __FASTCALL__ ix86_ArgFar(char * str,ix86Param *DisP)
   {
     seg = Use32Data ? *((short  *)(&DisP->RealCmd[5])) : *((short  *)(&DisP->RealCmd[3]));
     off = Use32Data ? *((long  *)(&DisP->RealCmd[1])) : (long)(*((short  *)(&DisP->RealCmd[1])));
-    newpos = ((long)(seg) << 4) + off;
-    if(FType == OLD_EXE) newpos += (((unsigned long)oldh.mz.mzHeaderSize) << 4);
+    newpos = ((long)(seg) << 4) + off; /* It is computing of x86 real-mode address */
   }
   DisP->codelen += Use32Data ? 6 : 4;
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
@@ -234,7 +295,7 @@ void __FASTCALL__ ix86_ArgNear(char * str,ix86Param *DisP)
   DisP->codelen += Use32Data ? 4 : 2;
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
   {
-    newpos = DisP->CodeAddress + lshift + DisP->codelen;
+    newpos = DisP->DisasmPrefAddr + lshift + DisP->codelen;
     disAppendFAddr(str,DisP->CodeAddress + 1,lshift,newpos,
                    Use32Data ? DISADR_NEAR32 : DISADR_NEAR16,0,DisP->codelen-1);
   }
@@ -260,8 +321,12 @@ static void __NEAR__ __FASTCALL__ getSIBRegs(char * base,char * scale,char * _in
   char ind = (code & 0x38) >> 3;
   tBool use32data = Use32Data;
   tBool useMMX = UseMMXSet;
+  tBool useXMMX = UseXMMXSet;
+  tBool brex, use64;
   Use32Data = True;
   UseMMXSet = False;
+  UseXMMXSet = False;
+  brex = use64 = 0;
   if(scl)
   {
     scale[0] = '*';
@@ -270,11 +335,33 @@ static void __NEAR__ __FASTCALL__ getSIBRegs(char * base,char * scale,char * _in
   }
   else scale[0] = 0;
   if(ind == 4) _index[0] = 0;
-  else         strcpy(_index,ix86_getREG(ind,True));
+  else 
+  {
+
+#ifdef IX86_64
+     if(x86_Bitness == DAB_USE64)
+     {
+         use64 = has67_in64?0:1;
+         brex = (k86_REX&2)>>1;
+     }
+#endif
+     strcpy(_index,k86_getREG(ind,True,brex,use64));
+  }
   if(bas == 5 && *mod == 0) { base[0] = 0; *mod = 2; }
-  else         strcpy(base,ix86_getREG(bas,True));
+  else
+  {
+#ifdef IX86_64
+     if(x86_Bitness == DAB_USE64)
+     {
+         use64 = has67_in64?0:1;
+         brex = (k86_REX&1)>>0;
+     }
+#endif
+    strcpy(base,k86_getREG(bas,True,brex,use64));
+  }
   Use32Data = use32data;
   UseMMXSet = useMMX;
+  UseXMMXSet = useXMMX;
 #if 0
   return ind == 4 || ind == 5 || bas == 4 || bas == 5; /** return is_stack */
 #endif
@@ -312,6 +399,13 @@ char * __FASTCALL__ ix86_getModRM(tBool w,unsigned char mod,unsigned char rm,ix8
  if(rm == 4 && Use32Addr) cptr = ConstrSibMod(ret1,base,_index,scale,DisP->RealCmd[2],&new_mode);
  else
  {
+#ifdef IX86_64
+   if(x86_Bitness == DAB_USE64)
+   {
+      cptr = !has67_in64 ? k86_QWordRegs[REX_reg(k86_REX&1,rm)] : k86_DWordRegs[REX_reg(k86_REX&1,rm)];
+   }
+   else
+#endif
    cptr = Use32Addr ? ix86_DWordRegs[rm] : ix86_A16[rm];
 #if 0
    is_stack = Use32Addr ? rm == 4 || rm == 5 : rm == 2 || rm == 3 || rm == 6;
@@ -371,6 +465,14 @@ char * __FASTCALL__ ix86_getModRM(tBool w,unsigned char mod,unsigned char rm,ix8
        case 2:
             {
               disp_long:
+#ifdef IX86_64
+		 if(x86_Bitness == DAB_USE64 && is_disponly)
+		 {
+		   strcat(ix86_modrm_ret,"rip");
+		   as_sign = True;
+		   is_disponly = False;
+		 }
+#endif
               if(!Use32Addr)
               {
                 strcat(ix86_modrm_ret,ix86_segpref);
@@ -390,8 +492,17 @@ char * __FASTCALL__ ix86_getModRM(tBool w,unsigned char mod,unsigned char rm,ix8
             break;
       default:
             {
+              tBool brex, use64;
               clen = 0;
-              strcat(ix86_modrm_ret,ix86_getREG(rm,w));
+              brex = use64 = 0;
+#ifdef IX86_64
+	      if(x86_Bitness == DAB_USE64)
+              {
+                brex = k86_REX&1;
+                use64 = Use64;
+              }
+#endif
+              strcat(ix86_modrm_ret,k86_getREG(rm,w,brex,use64));
             }
             break;
    }
@@ -416,8 +527,20 @@ static char * __NEAR__ __FASTCALL__ __getTile(ix86Param *DisP,tBool w,tBool d)
  char mod = (DisP->RealCmd[1] & 0xC0) >> 6;
  char rm  = (DisP->RealCmd[1] & 0x07);
  const char *regs,* modrm;
- regs = ix86_getREG(reg,w);
- if(mod == 3) modrm = ix86_getREG(rm,w);
+ tBool brex, use64;
+ brex = use64 = 0;
+#ifdef IX86_64
+ if(x86_Bitness == DAB_USE64)
+ {
+   brex = (k86_REX&4)>>2;
+   use64 = Use64;
+ }
+#endif
+ regs = k86_getREG(reg,w,brex,use64);
+#ifdef IX86_64
+ if(x86_Bitness == DAB_USE64) brex = k86_REX&1;
+#endif
+ if(mod == 3) modrm = k86_getREG(rm,w,brex,use64);
  else         modrm = ix86_getModRM(w,mod,rm,DisP);
  ix86_dtile[0] = 0;
  strcat(ix86_dtile,d ? regs : modrm);
@@ -507,7 +630,12 @@ void __FASTCALL__ ix86_SMov(char * str,ix86Param *DisP)
   sregptr = getSREG(sreg);
   DisP->codelen = 2;
   tileptr = ix86_getModRM(True,mod,reg,DisP);
-  if(sreg > 3) if((DisP->pro_clone & IX86_CPUMASK) < 3) DisP->pro_clone = IX86_CPU386;
+  if(sreg > 3)
+    if((DisP->pro_clone & IX86_CPUMASK) < 3)
+#ifdef IX86_64
+	if(x86_Bitness != DAB_USE64)
+#endif
+      DisP->pro_clone = IX86_CPU386;
   Use32Data = use32data;
   strcat(str,direct ? sregptr : tileptr);
   ix86_CStile(str,direct ? tileptr : sregptr);
@@ -525,7 +653,12 @@ void __FASTCALL__ ix86_ArgSegRM(char * str,ix86Param *DisP)
   strcat(str,sregptr);
   DisP->codelen++;
   tileptr = ix86_getModRM(True,mod,reg,DisP);
-  if(sreg > 3) if((DisP->pro_clone & IX86_CPUMASK) < 3) DisP->pro_clone = IX86_CPU386;
+  if(sreg > 3)
+    if((DisP->pro_clone & IX86_CPUMASK) < 3)
+#ifdef IX86_64
+	if(x86_Bitness != DAB_USE64)
+#endif
+      DisP->pro_clone = IX86_CPU386;
   Use32Data = use32data;
   ix86_CStile(str,tileptr);
 }
@@ -533,7 +666,16 @@ void __FASTCALL__ ix86_ArgSegRM(char * str,ix86Param *DisP)
 void __FASTCALL__ ix86_ArgAXDigit(char * str,ix86Param *DisP)
 {
   char w = DisP->RealCmd[0] & 0x01;
-  strcat(str,ix86_getREG(0,w));
+  tBool use64=0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64) use64 = Use64;
+  /* I guess that K9 will support "OP rax, imm64" forms
+     same as "OP rrx, imm64" since they are not longer
+     than 15 bytes in length
+     (example: "adc r13, 123456789ABCDEF0").
+     TODO: if(use64) w = -1; here */
+#endif
+  strcat(str,k86_getREG(0,w,0,use64));
   ix86_CStile(str,ix86_GetDigitTile(DisP,w,0,1));
 }
 
@@ -554,19 +696,49 @@ void __FASTCALL__ ix86_ArgIRegDigit(char * str,ix86Param *DisP)
 {
   char w = (DisP->RealCmd[0] >> 3 ) & 0x01;
   char reg = (DisP->RealCmd[0]) & 0x07;
-  strcat(str,ix86_getREG(reg,w));
+  tBool brex,use64;
+  brex = use64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64)
+  {
+    brex = k86_REX&1;
+    use64 = Use64;
+  }
+#endif
+  strcat(str,k86_getREG(reg,w,brex,use64));
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64 && Use64 && w) w = -1;
+#endif
   ix86_CStile(str,ix86_GetDigitTile(DisP,w,0,1));
 }
 
 void __FASTCALL__ ix86_ArgIReg(char *str,ix86Param *DisP)
 {
-  strcat(str,ix86_getREG(DisP->RealCmd[0] & 0x07,True));
+  tBool brex,use64;
+  brex = use64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64)
+  {
+    brex = k86_REX&1;
+    use64 = Use64;
+  }
+#endif
+  strcat(str,k86_getREG(DisP->RealCmd[0] & 0x07,True,brex,use64));
 }
 
 void __FASTCALL__ ix86_ArgAXIReg(char *str,ix86Param *DisP)
 {
-  strcat(str,ix86_getREG(0,True));
-  ix86_CStile(str,ix86_getREG(DisP->RealCmd[0] & 0x07,True));
+  tBool brex,use64;
+  brex = use64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64)
+  {
+    brex = k86_REX&1;
+    use64 = Use64;
+  }
+#endif
+  strcat(str,k86_getREG(0,True,0,use64));
+  ix86_CStile(str,k86_getREG(DisP->RealCmd[0] & 0x07,True,brex,use64));
 }
 
 void __FASTCALL__ ix86_ArgSInt(char *str,ix86Param *DisP)
@@ -574,13 +746,18 @@ void __FASTCALL__ ix86_ArgSInt(char *str,ix86Param *DisP)
   ix86_setModifier(str,Use32Data ? ix86_sizes[DWORD_PTR] : ix86_sizes[WORD_PTR]);
   if(!((DisP->flags & __DISF_SIZEONLY) == __DISF_SIZEONLY))
     disAppendDigits(str,DisP->CodeAddress + DisP->codelen,
-                 APREF_USE_TYPE,1,&DisP->RealCmd[DisP->codelen],DISARG_CHAR);
+                 APREF_USE_TYPE,1,&DisP->RealCmd[DisP->codelen],DISARG_CHAR | DISARG_IMM);
   DisP->codelen++;
 }
 
 void __FASTCALL__ ix86_ArgInt(char *str,ix86Param *DisP)
 {
-  Use32Data ? ix86_ArgDWord(str,DisP) : ix86_ArgWord(str,DisP);
+  tBool use64;
+  use64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64) use64 = Use64;
+#endif
+  use64 ? ix86_ArgQWord(str,DisP) : Use32Data ? ix86_ArgDWord(str,DisP) : ix86_ArgWord(str,DisP);
 }
 
 void __FASTCALL__ ix86_ArgRegRMDigit(char *str,ix86Param *DisP)
@@ -707,7 +884,7 @@ void __FASTCALL__ ix86_ArgGrp2(char *str,ix86Param *DisP)
    DisP->codelen++;
    strcpy(str,ix86_Gr2Names[code2]);
    /*
-      Added by "Kostya Nosov" <k-nosov@yandex.ru>: 
+      Added by "Kostya Nosov" <k-nosov@yandex.ru>:
       make NEEDREF_ALL for indirect "jmp" and "call"
    */
    oldDisNeedRef = disNeedRef;
@@ -718,6 +895,11 @@ void __FASTCALL__ ix86_ArgGrp2(char *str,ix86Param *DisP)
    else
    {
      wrd = code2 == 0x06 || code2 == 0x07 ? 0 : wrd; /** push / pop; */
+#ifdef IX86_64
+     if(x86_Bitness == DAB_USE64)
+	      sizptr = k86_REX & 0x08 ? QWORD_PTR : DWORD_PTR;
+     else
+#endif
      if(!wrd) sizptr = Use32Data ? DWORD_PTR : WORD_PTR;
      else     sizptr = Use32Data ? PWORD_PTR : DWORD_PTR;
      ix86_setModifier(str,ix86_sizes[sizptr]);
@@ -730,13 +912,38 @@ void __FASTCALL__ ix86_ArgAXMem(char *str,ix86Param *DisP)
 {
   unsigned char d = DisP->RealCmd[0] & 0x02;
   const char *mem,*reg;
-  mem = Use32Addr ? Get8SquareDig(1,DisP,False,True) : Get4SquareDig(1,DisP,False,True);
+  tBool use64;
+#ifdef IX86_64
+  unsigned char sav_REX;
+  sav_REX = k86_REX;
+#endif
+  use64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64 && Use64)
+  {
+    use64 = Use64;
+    mem = Get16SquareDig(1,DisP,False,True);
+  }
+  else
+#endif
+    mem = Use32Addr ? Get8SquareDig(1,DisP,False,True) : Get4SquareDig(1,DisP,False,True);
   strcpy(ix86_appstr,ix86_segpref);
   ix86_segpref[0] = 0;
   strcat(ix86_appstr,"[");
   strcat(ix86_appstr,mem);
   strcat(ix86_appstr,"]");
-  reg = ix86_getREG(0,DisP->RealCmd[0] & 0x01);
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64 && Use64) k86_REX=0;
+#endif
+  reg = k86_getREG(0,DisP->RealCmd[0] & 0x01,0,use64);
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64 && Use64) k86_REX=sav_REX;
+#endif
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64 && Use64)
+    DisP->codelen = 9;
+  else
+#endif
   DisP->codelen = Use32Addr ? 5 : 3;
   strcat(str,d ? ix86_appstr : reg);
   ix86_CStile(str,d ? reg : ix86_appstr);
@@ -746,7 +953,7 @@ void __FASTCALL__ ix86_InOut(char * str,ix86Param *DisP)
 {
   const char *arg,*reg1,*reg2,*regptr,*dig;
   char i;
-  regptr = ix86_getREG(0,DisP->RealCmd[0] & 0x01);
+  regptr = k86_getREG(0,DisP->RealCmd[0] & 0x01,0,0);
   dig = Get2Digit(DisP->RealCmd[1]);
   i = DisP->RealCmd[0] & 0x08;
   if(!i) DisP->codelen++;
@@ -771,15 +978,42 @@ void  __FASTCALL__ ix86_ExOpCodes(char *str,ix86Param *DisP)
 {
  unsigned char code;
  DisP->RealCmd = &DisP->RealCmd[1];
+ DisP->CodeAddress++;
  code = DisP->RealCmd[0];
  DisP->codelen++;
+#ifdef IX86_64
+ if(x86_Bitness == DAB_USE64)
+ {
+    strcpy(str,ix86_extable[code].name64);
+ }
+ else
+#endif
  strcpy(str,ix86_extable[code].name);
+#ifdef IX86_64
+ if(x86_Bitness == DAB_USE64)
+ {
+   if(ix86_extable[code].method64)
+   {
+     TabSpace(str,TAB_POS);
+     ix86_extable[code].method64(str,DisP);
+   }
+ }
+ else
+#endif
  if(ix86_extable[code].method)
  {
    TabSpace(str,TAB_POS);
    ix86_extable[code].method(str,DisP);
  }
- if(DisP->pro_clone < ix86_extable[code].pro_clone) DisP->pro_clone = ix86_extable[code].pro_clone;
+#ifdef IX86_64
+ if(x86_Bitness == DAB_USE64) DisP->pro_clone = ix86_extable[code].flags64;
+ else
+#endif
+ if((DisP->pro_clone&IX86_CPUMASK) < (ix86_extable[code].pro_clone&IX86_CPUMASK))
+ {
+	DisP->pro_clone &= ~IX86_CPUMASK;
+	DisP->pro_clone |= ix86_extable[code].pro_clone;
+ }
 }
 
 void  __FASTCALL__ ix86_ArgExGr0(char *str,ix86Param *DisP)
@@ -792,6 +1026,10 @@ void  __FASTCALL__ ix86_ArgExGr0(char *str,ix86Param *DisP)
     DisP->codelen++;
     ix86_setModifier(str,ix86_sizes[WORD_PTR]);
     strcat(str,ix86_getModRM(True,mod,reg,DisP));
+#ifdef IX86_64
+ if(x86_Bitness != DAB_USE64)
+#endif
+ if(rm < 4) DisP->pro_clone |= IX86_CPL0;
 }
 
 void  __FASTCALL__ ix86_ArgExGr1(char *str,ix86Param *DisP)
@@ -801,6 +1039,34 @@ void  __FASTCALL__ ix86_ArgExGr1(char *str,ix86Param *DisP)
     unsigned char mod = (DisP->RealCmd[1] & 0xC0) >> 6;
     unsigned char buf,ptr;
     tBool word;
+#ifdef IX86_64
+    if(x86_Bitness == DAB_USE64)
+    {
+      if(DisP->RealCmd[1] == 0xF8)
+      {
+        strcpy(str,"swapgs");
+        DisP->codelen++;
+	DisP->pro_clone |= K64_CPL0;
+        return;
+      }
+    }
+#endif
+    {
+      if(DisP->RealCmd[1] == 0xC8)
+      {
+        strcpy(str,"monitor");
+        DisP->codelen++;
+	DisP->pro_clone = IX86_P5CPU&~IX86_CPL0;
+        return;
+      }
+      if(DisP->RealCmd[1] == 0xC9)
+      {
+        strcpy(str,"mwait");
+        DisP->codelen++;
+	DisP->pro_clone = IX86_P5CPU&~IX86_CPL0;
+        return;
+      }
+    }
     strcpy(str,ix86_ExGrp1[rm]);
     TabSpace(str,TAB_POS);
     DisP->codelen++;
@@ -810,6 +1076,9 @@ void  __FASTCALL__ ix86_ArgExGr1(char *str,ix86Param *DisP)
     if(rm == 7)
     {
       word = False;
+#ifdef IX86_64
+      if(x86_Bitness != DAB_USE64)
+#endif
       DisP->pro_clone = IX86_CPU486;
       buf = 1;
       ptr = DUMMY_PTR;
@@ -828,10 +1097,19 @@ void  __FASTCALL__ ix86_ArgMovXRY(char *str,ix86Param *DisP)
     unsigned char reg = DisP->RealCmd[1] & 0x07;
     unsigned char ridx = (code & 0x01) + ((code >> 1) & 0x02);
     unsigned char d = (code >> 1) & 0x01;
+    tBool brex,use64;
+    brex = use64 = 0;
+#ifdef IX86_64
+    if(x86_Bitness == DAB_USE64)
+    {
+       brex = k86_REX & 1;
+       use64 = Use64;
+    }
+#endif
     DisP->codelen++;
     Use32Data = True;
-    strcat(str,d ? xry[ridx][xreg] : ix86_getREG(reg,True));
-    ix86_CStile(str,d ? ix86_getREG(reg,True) : xry[ridx][xreg]);
+    strcat(str,d ? xry[ridx][xreg] : k86_getREG(reg,True,brex,use64));
+    ix86_CStile(str,d ? k86_getREG(reg,True,brex,use64) : xry[ridx][xreg]);
 }
 
 void  __FASTCALL__ ix86_DblShift(char *str,ix86Param *DisP)
@@ -860,10 +1138,23 @@ static void __NEAR__ __FASTCALL__ ix86_ArgxMM(char *str,ix86Param *DisP,tBool di
    else
    {
       const char *mmx,*exx;
-      mmx = ix86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True);
+      tBool brex,use64;
+      brex = use64 = 0;
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64)
+      {
+         brex = (k86_REX&4)>>2;
+         use64 = Use64;
+         if(brex && !as_xmmx) brex = 0; /* Note: there are only 8 mmx registers */
+      }
+#endif
+      mmx = k86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True,brex,use64);
       if(as_xmmx) UseXMMXSet = False;
       else        UseMMXSet = False;
-      exx = ix86_getREG(DisP->RealCmd[1] & 0x07,True);
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64) brex = k86_REX&1;
+#endif
+      exx = k86_getREG(DisP->RealCmd[1] & 0x07,True,brex,use64);
       strcat(str,direct ? exx : mmx);
       strcat(str,",");
       strcat(str,direct ? mmx : exx);
@@ -884,10 +1175,23 @@ static void __NEAR__ __FASTCALL__ ix86_ArgxMMRev(char *str,ix86Param *DisP,tBool
    else
    {
       const char *mmx,*exx;
-      mmx = ix86_getREG(DisP->RealCmd[1] & 0x07,True);
+      tBool brex,use64;
+      brex = use64 = 0;
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64)
+      {
+         brex = k86_REX&1;
+         use64 = Use64;
+         if(brex && !as_xmmx) brex = 0; /* Note: there are only 8 mmx registers */
+      }
+#endif
+      mmx = k86_getREG(DisP->RealCmd[1] & 0x07,True,brex,use64);
       if(as_xmmx) UseXMMXSet = False;
       else        UseMMXSet = False;
-      exx = ix86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True);
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64) brex = (k86_REX&4)>>2;
+#endif
+      exx = k86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True,brex,use64);
       strcat(str,direct ? exx : mmx);
       strcat(str,",");
       strcat(str,direct ? mmx : exx);
@@ -897,12 +1201,22 @@ static void __NEAR__ __FASTCALL__ ix86_ArgxMMRev(char *str,ix86Param *DisP,tBool
 
 static void __NEAR__ __FASTCALL__ ix86_ArgxMMXGroup(char *str,const char *name,ix86Param *DisP,tBool as_xmmx)
 {
+  tBool brex,use64;
+  brex = use64 = 0;
+#ifdef IX86_64
+  if(x86_Bitness == DAB_USE64)
+  {
+    brex = k86_REX&1;
+    use64 = Use64;
+    if(brex && !as_xmmx) brex = 0; /* Note: there are only 8 mmx registers */
+  }
+#endif
   DisP->codelen++;
   strcpy(str,name);
   TabSpace(str,TAB_POS);
   if(as_xmmx) UseXMMXSet = True;
   else        UseMMXSet = True;
-  strcat(str,ix86_getREG(DisP->RealCmd[1] & 0x07,True));
+  strcat(str,k86_getREG(DisP->RealCmd[1] & 0x07,True,brex,use64));
   if(as_xmmx) UseXMMXSet = False;
   else        UseMMXSet = False;
   strcat(str,",");
@@ -923,9 +1237,21 @@ static void __NEAR__ __FASTCALL__ ix86_ArgRXMM(char *str,ix86Param *DisP,tBool d
    else
    {
       const char *mmx,*exx;
-      mmx = ix86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True);
+      tBool brex,use64;
+      brex = use64 = 0;
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64)
+      {
+        brex = (k86_REX&4)>>2;
+        use64 = Use64;
+      }
+#endif
+      mmx = k86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True,brex,use64);
       UseXMMXSet = False;
-      exx = ix86_getREG(DisP->RealCmd[1] & 0x07,True);
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64) brex = k86_REX&1;
+#endif
+      exx = k86_getREG(DisP->RealCmd[1] & 0x07,True,brex,use64);
       strcat(str,direct ? exx : mmx);
       strcat(str,",");
       strcat(str,direct ? mmx : exx);
@@ -942,12 +1268,25 @@ static void __NEAR__ __FASTCALL__ ix86_ArgRxMMRev(char *str,ix86Param *DisP,tBoo
    else
    {
       const char *mmx,*exx;
+      tBool brex,use64;
+      brex = use64 = 0;
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64)
+      {
+        brex = k86_REX&1;
+        use64 = Use64;
+        if(brex && !as_xmmx) brex = 0; /* Note: there are only 8 mmx registers */
+      }
+#endif
       if(as_xmmx) UseXMMXSet = True;
       else        UseMMXSet = True;
-      mmx = ix86_getREG(DisP->RealCmd[1] & 0x07,True);
+      mmx = k86_getREG(DisP->RealCmd[1] & 0x07,True,brex,True);
       if(as_xmmx) UseXMMXSet = False;
       else        UseMMXSet = False;
-      exx = ix86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True);
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64) brex = (k86_REX&4)>>2;
+#endif
+      exx = k86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True,brex,use64);
       strcat(str,direct ? mmx : exx);
       strcat(str,",");
       strcat(str,direct ? exx : mmx);
@@ -968,7 +1307,17 @@ static void __NEAR__ __FASTCALL__ ix86_ArgxMMxMM(char *str,ix86Param *DisP,tBool
    else
    {
       const char *mmx,*exx;
-      mmx = ix86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True);
+      tBool brex,use64;
+      brex = use64 = 0;
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64)
+      {
+        brex = (k86_REX&4)>>2;
+        use64 = Use64;
+        if(brex && !xmmx_first) brex = 0; /* Note: there are only 8 mmx registers */
+      }
+#endif
+      mmx = k86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True,brex,use64);
       if(xmmx_first)
       {
         UseXMMXSet = False;
@@ -979,7 +1328,10 @@ static void __NEAR__ __FASTCALL__ ix86_ArgxMMxMM(char *str,ix86Param *DisP,tBool
         UseXMMXSet = True;
         UseMMXSet = False;
       }
-      exx = ix86_getREG(DisP->RealCmd[1] & 0x07,True);
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64) brex = k86_REX&1;
+#endif
+      exx = k86_getREG(DisP->RealCmd[1] & 0x07,True,brex,use64);
       if(xmmx_first) UseMMXSet = False;
       else           UseXMMXSet = False;
       strcat(str,direct ? exx : mmx);
@@ -1123,9 +1475,18 @@ void __FASTCALL__ ix86_ArgMovYX(char *str,ix86Param *DisP)
       const char *dst,*src;
       unsigned char mod;
       tBool u32data;
+      tBool brex,use64;
+      brex = use64 = 0;
+#ifdef IX86_64
+      if(x86_Bitness == DAB_USE64)
+      {
+        brex = (k86_REX&4)>>2;
+        use64 = Use64;
+      }
+#endif
       mod = (DisP->RealCmd[1] >> 6) & 0x03;
       ix86_setModifier(str,DisP->RealCmd[0] & 0x01 ? Use32Data ? ix86_sizes[DWORD_PTR] : ix86_sizes[WORD_PTR] : ix86_sizes[BYTE_PTR]);
-      dst = ix86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True);
+      dst = k86_getREG((DisP->RealCmd[1] >> 3) & 0x07,True,brex,use64);
       u32data = Use32Data;
       Use32Data = False;
       src = ix86_getModRM(DisP->RealCmd[0] & 0x01,mod,DisP->RealCmd[1] & 0x07,DisP);
@@ -1159,7 +1520,11 @@ void __FASTCALL__ ix86_ArgKatmaiGrp1(char *str,ix86Param *DisP)
      ix86_ArgMod(str,DisP);
    }
    else DisP->codelen++;
-   if(code2 == 5 || code2 == 6) DisP->pro_clone = IX86_P4MMX;
+   if(code2 == 5 || code2 == 6)
+#ifdef IX86_64
+	if(x86_Bitness != DAB_USE64)
+#endif
+      DisP->pro_clone = IX86_P4MMX;
 }
 
 void __FASTCALL__ ix86_ArgKatmaiGrp2(char *str,ix86Param *DisP)
@@ -1167,7 +1532,7 @@ void __FASTCALL__ ix86_ArgKatmaiGrp2(char *str,ix86Param *DisP)
    unsigned char code2 = ( DisP->RealCmd[1] & 0x38 ) >> 3;
    strcpy(str,ix86_KatmaiGr2Names[code2]);
    TabSpace(str,TAB_POS);
-   ix86_ArgMod(str,DisP);
+   ix86_ArgModB(str,DisP);
 }
 
 void __FASTCALL__  ix86_ArgXMMRMDigit(char *str,ix86Param *DisP)
@@ -1182,6 +1547,33 @@ void __FASTCALL__  ix86_ArgXMMRMDigit(char *str,ix86Param *DisP)
    UseXMMXSet = False;
 }
 
+void __FASTCALL__  ix86_ArgXMMCmp(char *str,ix86Param *DisP)
+{
+   char * a1,*a2;
+   unsigned char suffix;
+   char name[6], realname[10];
+   UseXMMXSet = True;
+   DisP->codelen++;
+   a1 = ix86_getTileWD(DisP);
+   suffix = DisP->RealCmd[DisP->codelen-1];
+   if(suffix < 8)
+   {
+    /*Note: this code suppose that name is cmpXY*/
+      strncpy(name, str, 5);
+      name[5] = 0;
+      strncpy(realname,name,3);
+      realname[3] = 0;
+      strcat(realname,ix86_KatmaiCmpSuffixes[suffix]);
+      strcat(realname, &name[3]);
+      strcpy(str, realname);
+      TabSpace(str, TAB_POS);
+   }
+   strcat(str,a1);
+   a2 = ix86_GetDigitTile(DisP,0,0,DisP->codelen-1);
+   if(!(suffix < 8)) ix86_CStile(str,a2);
+   UseXMMXSet = False;
+}
+
 void __FASTCALL__  ix86_ArgMMRMDigit(char *str,ix86Param *DisP)
 {
    char * a1,*a2;
@@ -1238,8 +1630,19 @@ void __FASTCALL__ ix86_3dNowOpCodes( char *str,ix86Param *DisP)
  strcpy(str,ix86_3dNowtable[code].name);
  TabSpace(str,TAB_POS);
  strcat(str,ix86_Katmai_buff);
- DisP->pro_clone = ix86_3dNowtable[code].pro_clone;
+#ifdef IX86_64
+   if(x86_Bitness == DAB_USE64) DisP->pro_clone = K64_ATHLON|K64_MMX;
+   else
+#endif
+   DisP->pro_clone = ix86_3dNowtable[code].pro_clone;
+}
+
+void __FASTCALL__ ix86_3dNowPrefetchGrp(char *str,ix86Param *DisP)
+{
+   unsigned char mod = ( DisP->RealCmd[1] & 0x38 ) >> 3;
+   strcpy(str,ix86_3dPrefetchGrp[mod]);
+   TabSpace(str,TAB_POS);
+   ix86_ArgModB(str,DisP);
 }
 
 
-
