@@ -3,7 +3,7 @@ void disable_misalign( void )
 /* Disable misaligned memory access */
   asm("pushfl\n"
       "	popl	%%eax\n"
-      "	orl	$0x40000, %%eax\n"
+      "	bts	$18, %%eax\n"
       "	pushl	%%eax\n"
       "	popfl"::);
 /* THIS CODE MUST PRODUCED AN EXCEPTION */
